/***************************************************************************
 *            m_book-fun.c
 *
 *  Mon Oct 31 22:06:12 2005
 *  Copyright  2005  Tomek Jankowski
 *  Email	tomcio-j@o2.pl
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
/*=====================================================================
 *	Plik zawiera funkcje zwiazane tylko z obsluga ksiazek wielotomowych
 *====================================================================*/
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glade/glade.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <glib/gi18n-lib.h>
#include <stdio.h>
#include <string.h>

#include "m_book-fun.h"
#include "m_volume-fun.h"
#include "database-fun.h"
#include "interface-fun.h"

#define VERY_SHORT_STRING	12
#define SHORT_STRING		128
#define MEDIUM_STRING		256
#define	LONG_STRING			1024
#define	DESCRIPTION_STRING	2048
#define	VERY_LONG_STRING	6144

/*
 *	Funkcja pobiera wartosci wszystkich pol i przekazuje je innym funkcjom w
 *	postaci struktury zdefiniowanej w naglowku 'm_book-fun.h'. W ramach 
 *	przypomnienia powiem, ze dane zostaly juz sprawdzone przez funkcje 
 *	'check_book_data' zdefiniowana w pliku book-fun.h
 */

MultiVolSharedDataStruc	*m_get_data_from_win	(GtkWidget	*widget)	{
	MultiVolSharedDataStruc	*book_data;
	GladeXML				*xml;
	GtkTextBuffer			*text_buffer;
	GtkTextIter				text_start;	
	GtkTextIter				text_end;
	GtkWidget				*tmp;
	gint					status;
	gint					result;
	gchar					sql_query	[SHORT_STRING];
	sqlite3_stmt			*db_row;
	
	xml = glade_get_widget_tree (widget);
	book_data = g_new (MultiVolSharedDataStruc, 1);
	g_printf (">>> Getting values from 'm_book_win...\n");
	
	/*	Pobieramy ID ksiazki	*/
	tmp = glade_xml_get_widget (xml, "book_id_lb");
	if (strlen (gtk_label_get_text (GTK_LABEL (tmp))) > 0)	{
		sscanf (gtk_label_get_text (GTK_LABEL (tmp)), "%d", &book_data->id);
	}
	
	/*	Pobieramy wartosc pola 'title_en'	*/
	tmp = glade_xml_get_widget (xml, "title_en");
	g_sprintf (book_data->title, "%s", gtk_entry_get_text (GTK_ENTRY (tmp)));
	
	/*	Pobieramy wartosc pola 'genere_en'	*/
	tmp = glade_xml_get_widget (xml, "genere_en");
	if (strcmp (gtk_entry_get_text (GTK_ENTRY (tmp)), "") == 0)	{
		book_data->genere = 0;
	}
	else	{
		book_data->genere = get_genere_id (gtk_entry_get_text (GTK_ENTRY (tmp)));
	}
	
	/*	Pobieramy wartosc pola 'publish_house_en'	*/
	tmp = glade_xml_get_widget (xml, "publish_house_en");
	if (strcmp (gtk_entry_get_text (GTK_ENTRY (tmp)), "") == 0)	{
		book_data->publish_house = 0;
	}
	else	{
		book_data->publish_house = get_publish_house_id (gtk_entry_get_text (GTK_ENTRY (tmp)));
	}
	
	/*	Pobieramy wartosc pola 'language_en'	*/
	tmp = glade_xml_get_widget (xml, "language_en");
	if (strlen ( gtk_entry_get_text (GTK_ENTRY (tmp))) == 0)	{
		book_data->language = 0;
	}
	else	{
		book_data->language = get_language_id (gtk_entry_get_text (GTK_ENTRY (tmp)));
	}
	
	/*	Pobieramy wartosc pola 'm_first_author_en'	*/
	tmp = glade_xml_get_widget (xml, "m_first_author_en");
	if (strlen ( gtk_entry_get_text (GTK_ENTRY (tmp))) == 0)	{
		book_data->first_author = 0;
	}
	else	{
		book_data->first_author = get_author_id (gtk_entry_get_text (GTK_ENTRY (tmp)));
		
		/*	Pobieramy wartosc pola 'second_author_en'	*/
		tmp = glade_xml_get_widget (xml, "second_author_en");
		if (strlen ( gtk_entry_get_text (GTK_ENTRY (tmp))) == 0)	{
			book_data->second_author = 0;
		}
		else	{
			book_data->second_author = get_author_id (gtk_entry_get_text (GTK_ENTRY (tmp)));
		}
		
		/*	Pobieramy wartosc pola 'third_author_en'	*/
		tmp = glade_xml_get_widget (xml, "third_author_en");
		if (strlen ( gtk_entry_get_text (GTK_ENTRY (tmp))) == 0)	{
			book_data->third_author = 0;
		}
		else	{
			book_data->third_author = get_author_id (gtk_entry_get_text (GTK_ENTRY (tmp)));
		}
	}
	
	/*	Pobieramy wartosc pola 'orginal_title_en'	*/
	tmp = glade_xml_get_widget (xml, "orginal_title_en");
	if (strlen ( gtk_entry_get_text (GTK_ENTRY (tmp))) == 0)	{
		g_sprintf (book_data->orginal_title, "%s", "");
	}
	else	{
		g_sprintf (book_data->orginal_title, "%s", gtk_entry_get_text (GTK_ENTRY (tmp)));
	}
	
	/*	Pobieramy wartosc pola 'orginal_language_en'	*/
	tmp = glade_xml_get_widget (xml, "orginal_language_en");
	if (strlen ( gtk_entry_get_text (GTK_ENTRY (tmp))) == 0)	{
		book_data->orginal_language = 0;
	}
	else	{
		book_data->orginal_language = get_language_id (gtk_entry_get_text (GTK_ENTRY (tmp)));
	}
	
	tmp = glade_xml_get_widget (xml, "m_book_same_translators_chbox");
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
		/*	Ustawiamy wspolnych tlumaczy	*/
		book_data->same_translators = 1;
		/*	Pobieramy wartosc pola 'm_first_translator_en'	*/
		tmp = glade_xml_get_widget (xml, "m_same_first_translator_en");
		if (strlen ( gtk_entry_get_text (GTK_ENTRY (tmp))) == 0)	{
				book_data->first_translator = 0;
		}
		else	{
			book_data->first_translator = get_translator_id (gtk_entry_get_text (GTK_ENTRY (tmp)));
			
			/*	Pobieramy wartosc pola 'same_second_translator_en'	*/
			tmp = glade_xml_get_widget (xml, "same_second_translator_en");
			if (strlen ( gtk_entry_get_text (GTK_ENTRY (tmp))) == 0)	{
				book_data->second_translator = 0;
			}
			else	{
				book_data->second_translator = get_translator_id (gtk_entry_get_text (GTK_ENTRY (tmp)));
			}
			
			/*	Pobieramy wartosc pola 'third_translator_en'	*/
			tmp = glade_xml_get_widget (xml, "same_third_translator_en");
			if (strlen ( gtk_entry_get_text (GTK_ENTRY (tmp))) == 0)	{
				book_data->third_translator = 0;
			}
			else	{
				book_data->third_translator = get_translator_id (gtk_entry_get_text (GTK_ENTRY (tmp)));
			}
		}
	}
	else	{
		book_data->same_translators = 0;
		book_data->first_translator = 0;
		book_data->second_translator = 0;
		book_data->third_translator = 0;
	}

	tmp = glade_xml_get_widget (xml, "m_book_same_publish_year_check");
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
		tmp = glade_xml_get_widget (xml, "same_publish_year_spin");
		book_data->same_publish_year = 1;
		book_data->publish_year = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tmp));
	}
	else	{
		book_data->same_publish_year = 0;	
		book_data->publish_year = 0;
	}
	
	/*	Pobieramy wartosc pola 'description_text'	*/
	tmp = glade_xml_get_widget (xml, "description_text");
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tmp));
	gtk_text_buffer_get_bounds (text_buffer, &text_start, &text_end);
	g_sprintf (book_data->description, "%s", gtk_text_buffer_get_text (text_buffer, &text_start, &text_end, TRUE));
	
	return	book_data;
}

/*
 *	Funkcja przygotowuje okno dodawania/edycji do uzycia ladujac np. listy autorow, 
 *	gatunkow itd. do odpowiednuch widgetow GtkEntry w oknie.
 */

void		m_prepare_book_win	(GtkWidget	*widget)	{
	GladeXML			*xml;
	GtkListStore		*list;
	GtkTreeIter			iter;
	GtkEntryCompletion	*completion;
	GtkWidget			*tmp;
	gint				status;
	gint				rows;
	sqlite3_stmt		*db_row;
	
	xml = glade_get_widget_tree (widget);	
	g_printf (">>> Preparing window...\n");
	
	/*	Ladujemy autorow	*/
	completion = gtk_entry_completion_new ();
	gtk_entry_completion_set_inline_completion (completion, TRUE);
	gtk_entry_completion_set_text_column (completion, 0);
	
	rows = count_returned_rows ("SELECT COUNT(id) FROM authors;");
	g_printf ("\t<<< Loading %d authors...\n", rows);
	if (rows > 0)	{
		sqlite3_prepare (database, "SELECT author FROM authors;", -1, &db_row, 0);
		status = sqlite3_step (db_row);
		g_printf ("\t<<< Loading table \'authors\', SQLite result: %d\n", status);
	
		list = gtk_list_store_new (1, G_TYPE_STRING);
		if (status == 101)	{
			gtk_list_store_append (list, &iter);
			gtk_list_store_set (list, &iter,
								0, sqlite3_column_text (db_row, 0),
								-1);
		}
		else	{
			while (status == 100)	{
				gtk_list_store_append (list, &iter);
				gtk_list_store_set (list, &iter,
									0, sqlite3_column_text (db_row, 0),
									-1);
				status = sqlite3_step (db_row);
			}
		}
		
		sqlite3_finalize (db_row);
		
		gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (list));	
		
		/*	W koncu ladujemy listy do widgetow...	*/
		/*	Nazwa pola pierwszego autora jest inna w roznych oknach	*/
		tmp = glade_xml_get_widget (xml, "m_first_author_en");	
		gtk_entry_set_completion (GTK_ENTRY (tmp), completion);
		
		tmp = glade_xml_get_widget (xml, "second_author_en");
		completion = gtk_entry_completion_new ();
		gtk_entry_completion_set_inline_completion (completion, TRUE);
		gtk_entry_completion_set_text_column (completion, 0);
		gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (list));	
		gtk_entry_set_completion (GTK_ENTRY (tmp), completion);
		
		tmp = glade_xml_get_widget (xml, "third_author_en");	
		completion = gtk_entry_completion_new ();
		gtk_entry_completion_set_inline_completion (completion, TRUE);
		gtk_entry_completion_set_text_column (completion, 0);
		gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (list));	
		gtk_entry_set_completion (GTK_ENTRY (tmp), completion);
	}
	
	/* GtkEntry 'genere_en'   */
	completion = gtk_entry_completion_new ();
	gtk_entry_completion_set_inline_completion (completion, TRUE);
	gtk_entry_completion_set_text_column (completion, 0);
	
	rows = count_returned_rows ("SELECT COUNT(id) FROM generes;");
	g_printf ("\t<<< Loading %d genres...\n", rows);
	if (rows > 0)	{
		sqlite3_prepare (database, "SELECT genere FROM generes;", -1, &db_row, 0);
		status = sqlite3_step (db_row);
		g_printf ("\t<<< Loading table \'genres\', SQLite result: %d\n", status);
		
		list = gtk_list_store_new (1, G_TYPE_STRING);
		if (status == 101)	{
			gtk_list_store_append (list, &iter);
			gtk_list_store_set (list, &iter,
								0, sqlite3_column_text (db_row, 0),
								-1);
		}
		else	{
			while (status == 100)	{
				gtk_list_store_append (list, &iter);
				gtk_list_store_set (list, &iter,
									0, sqlite3_column_text (db_row, 0),
									-1);
				status = sqlite3_step (db_row);
			}
		}
		
		sqlite3_finalize (db_row);
		
		gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (list));

		tmp = glade_xml_get_widget (xml, "genere_en");	
		gtk_entry_set_completion (GTK_ENTRY (tmp), completion);;
	}
	
	/*	Ladujemy wydawnictwa	*/
	completion = gtk_entry_completion_new ();
	gtk_entry_completion_set_inline_completion (completion, TRUE);
	gtk_entry_completion_set_text_column (completion, 0);
	
	rows = count_returned_rows ("SELECT COUNT(id) FROM publish_houses;");
	g_printf ("\t<<< Loading %d publish houses...\n", rows);
	if (rows > 0)	{
		sqlite3_prepare (database, "SELECT publish_house FROM publish_houses;", -1, &db_row, 0);
		status = sqlite3_step (db_row);
		g_printf ("\t<<< Loading table \'publish_houses\', SQLite result: %d\n", status);
		
		list = gtk_list_store_new (1, G_TYPE_STRING);
		if (status == 101)	{
			gtk_list_store_append (list, &iter);
			gtk_list_store_set (list, &iter,
								0, sqlite3_column_text (db_row, 0),
								-1);
		}
		else	{
			while (status == 100)	{
				gtk_list_store_append (list, &iter);
				gtk_list_store_set (list, &iter,
									0, sqlite3_column_text (db_row, 0),
									-1);
				status = sqlite3_step (db_row);
			}
		}
		
		sqlite3_finalize (db_row);
		
		gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (list));	
		
		tmp = glade_xml_get_widget (xml, "publish_house_en");	
		gtk_entry_set_completion (GTK_ENTRY (tmp), completion);;
	}
	
	/*	Ladujemy jezyki	*/
	completion = gtk_entry_completion_new ();
	gtk_entry_completion_set_inline_completion (completion, TRUE);
	gtk_entry_completion_set_text_column (completion, 0);
	
	rows = count_returned_rows ("SELECT COUNT(id) FROM languages;");
	g_printf ("\t<<< Loading %d languages...\n", rows);
	if (rows > 0)	{
		sqlite3_prepare (database, "SELECT language FROM languages;", -1, &db_row, 0);
		status = sqlite3_step (db_row);
		g_printf ("\t<<< Loading table \'languages\', SQLite result: %d\n", status);
		
		list = gtk_list_store_new (1, G_TYPE_STRING);
		if (status == 101)	{
			gtk_list_store_append (list, &iter);
			gtk_list_store_set (list, &iter,
								0, sqlite3_column_text (db_row, 0),
								-1);
		}
		else	{
			while (status == 100)	{
				gtk_list_store_append (list, &iter);
				gtk_list_store_set (list, &iter,
									0, sqlite3_column_text (db_row, 0),
									-1);
				status = sqlite3_step (db_row);
			}
		}
		
		sqlite3_finalize (db_row);
		
		gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (list));	
		
		tmp = glade_xml_get_widget (xml, "language_en");	
		gtk_entry_set_completion (GTK_ENTRY (tmp), completion);
		
		tmp = glade_xml_get_widget (xml, "orginal_language_en");	
		completion = gtk_entry_completion_new ();
		gtk_entry_completion_set_inline_completion (completion, TRUE);
		gtk_entry_completion_set_text_column (completion, 0);
		gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (list));	
		gtk_entry_set_completion (GTK_ENTRY (tmp), completion);
	}
	
	/*	Ladujemy tlumaczy	*/
	completion = gtk_entry_completion_new ();
	gtk_entry_completion_set_inline_completion (completion, TRUE);
	gtk_entry_completion_set_text_column (completion, 0);
	
	rows = count_returned_rows ("SELECT COUNT(id) FROM translators;");
	g_printf ("\t<<< Loading %d translators...\n", rows);
	if (rows > 0)	{
		sqlite3_prepare (database, "SELECT translator FROM translators;", -1, &db_row, 0);
		status = sqlite3_step (db_row);
		g_printf ("\t<<< Loading table \'translators\', SQLite result: %d\n", status);
	
		list = gtk_list_store_new (1, G_TYPE_STRING);
		if (status == 101)	{
			gtk_list_store_append (list, &iter);
			gtk_list_store_set (list, &iter,
								0, sqlite3_column_text (db_row, 0),
								-1);
		}
		else	{
			while (status == 100)	{
				gtk_list_store_append (list, &iter);
				gtk_list_store_set (list, &iter,
									0, sqlite3_column_text (db_row, 0),
									-1);
				status = sqlite3_step (db_row);
			}
		}
		
		sqlite3_finalize (db_row);
		
		gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (list));	
		
		/*	W koncu ladujemy listy do widgetow...	*/
		/*	Nazwa pola pierwszego tlumacza jest inna w roznych oknach	*/
		tmp = glade_xml_get_widget (xml, "m_same_first_translator_en");	
		gtk_entry_set_completion (GTK_ENTRY (tmp), completion);
		
		tmp = glade_xml_get_widget (xml, "same_second_translator_en");	
		completion = gtk_entry_completion_new ();
		gtk_entry_completion_set_inline_completion (completion, TRUE);
		gtk_entry_completion_set_text_column (completion, 0);
		gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (list));	
		gtk_entry_set_completion (GTK_ENTRY (tmp), completion);
		
		tmp = glade_xml_get_widget (xml, "same_third_translator_en");	
		completion = gtk_entry_completion_new ();
		gtk_entry_completion_set_inline_completion (completion, TRUE);
		gtk_entry_completion_set_text_column (completion, 0);
		gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (list));	
		gtk_entry_set_completion (GTK_ENTRY (tmp), completion);
	}
	set_widget_sensinitive (widget, "m_book_hbox", TRUE);	
}

/*
 *	Funkcja sprawdza poprawnosc wprowadzonych danych do okna dodawania/edycji 
 *	ksiazki. Jezeli dane sa bledne lub jezeli ich nie ma zwracana jest wartosc 
 *	FALSE i wyswietlany jest komunikat z ostrzezeniem.
 *	Sprawdzane sa tylko te dane, ktore sa niezbedne do zapisania ksiazki w bazie 
 *	danych, czyli: tytul i jezyk.
 *	Funkcja sprawdza czy takie pola jak: "Publish house", "Genere", "Author", 
 *	"Translator", "Pages noumber" i "Publish year" sa puste. Jezeli sa zostaje
 *	wyswietlony kominukat z informacja. Poniewaz te pola nie sa wymagane, zostanie
 *	zwrocona wartosc TRUE.
 *	
 *	Przy okazji sprawdzamy, czy pole 'Description' nie zawiera zbyt dlugiego opisu
 *	maksymlna dlugosc to 2000 znakow.
 */

gboolean	m_check_book_data	(GtkWidget	*widget)	{
	GladeXML		*xml;
	GtkTreeModel	*model;
  	GtkTreeIter		iter;	
	GtkTextBuffer 	*text_buffer;
	GtkTextIter		text_start;	
	GtkTextIter		text_end;
	GtkWidget		*tmp;
	GtkWidget		*genere_en;
	GtkWidget		*publish_house_en;
	GtkWidget		*first_author_en;
	GtkWidget		*first_translator_en;
	GtkWidget		*language_en;
	gint			volumes_number;
	gboolean		result;
	
	xml = glade_get_widget_tree (widget);
	/*	Pole 'Title' nie moze byc puste	*/
	tmp = glade_xml_get_widget (xml, "title_en");
	if (strlen ( gtk_entry_get_text (GTK_ENTRY (tmp))) == 0)	{
		show_message_dlg (_("Invalid field value"), _("Field \'Title\' can't be empty!"), 1);
		return FALSE;
	}
	
	/*	Sprawdzamy ile ksiazek zostalo dodanych do listy, minimalnie na liscie
	 *	musza znajdowac sie dwie ksiazki zeby ksiazka zostala dodana	*/
	tmp = glade_xml_get_widget (xml, "books_list");
	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tmp));
	
	result = gtk_tree_model_get_iter_first (model, &iter);	
	if (result == FALSE)	{
		show_message_dlg (_("Empty list"), _("You must add minimum two volumes "
					"to the list!"), 1);
		return FALSE;
	}
	else	{
		result = gtk_tree_model_iter_next (model, &iter);	
		if (result == FALSE)	{
			show_message_dlg (_("Too few volumes"), _("You must add minimum two volumes "
					"to the list!"), 1);
		
			return FALSE;
		}
	}
	
	/*	Sprawdzamy dlugosc testu w polu 'Description'	*/
	tmp = glade_xml_get_widget (xml, "description_text");
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tmp));
	gtk_text_buffer_get_bounds (text_buffer, &text_start, &text_end);	
	if (strlen ( gtk_text_buffer_get_text (text_buffer, &text_start, &text_end, TRUE)) > 2000)	{
		show_message_dlg (_("Invalid field value"), _("Description can't be longer than 2000 signs!"), 1);
		return FALSE;
	}
	
	/*	Sprawdzamy czy opcjonalne pola zostaly wypelnione */
	genere_en = glade_xml_get_widget (xml, "genere_en");
	publish_house_en = glade_xml_get_widget (xml, "publish_house_en");
	first_author_en = glade_xml_get_widget (xml, "m_first_author_en");
	language_en = glade_xml_get_widget (xml, "language_en");

	if (strlen ( gtk_entry_get_text (GTK_ENTRY (genere_en))) == 0 ||
		strlen ( gtk_entry_get_text (GTK_ENTRY (publish_house_en))) == 0 ||
		strlen ( gtk_entry_get_text (GTK_ENTRY (first_author_en))) == 0 ||
		strlen ( gtk_entry_get_text (GTK_ENTRY (language_en))) == 0)		
	{
		show_message_dlg (_("Hint"), _("Some of additional fields (like \'Author\', "
						"\'Publish house\' or \'Genere\') are empty. There aren't "
						"required, but for better books organisation they should be "
						"filled in future."), 0);
	}
	
	return TRUE;
}

/*
 *	Funkcja sprawdza, czy wartosc pol takich jak np.: 'Language', 'Genere' itp.
 *	znajduje sie juz w bazie danych. Jezeli owa wartosc nie istnieje w bazie 
 *	dancyh, user otrzymuje propozycje dodania tej wartosci wprost z okna
 *	dodawania/edycji ksiazki, a nie poprzez okno 'menage_db_win'
 */

void	m_check_linked_book_data		(GtkWidget		*widget)	{
	GladeXML			*xml;
	GtkWidget			*tmp;
	gboolean			result;
	gint				status;
	gint				rows;
	gchar				sql_query	[SHORT_STRING];
	gchar				string		[MEDIUM_STRING];
	
	sqlite3_stmt		*db_row;
	
	xml = glade_get_widget_tree (widget);
	
	/* Sprawdzamy pole 'Genere'	*/
	tmp = glade_xml_get_widget (xml, "genere_en");
	
	/*	Jezeli pole jest puste to je pomijamy	*/
	if (strcmp ( gtk_entry_get_text (GTK_ENTRY (tmp)), "") > 0)	{
		g_sprintf (sql_query, "SELECT COUNT(id) FROM generes WHERE genere=\'%s\';",
					gtk_entry_get_text (GTK_ENTRY (tmp)));
		rows = count_returned_rows (sql_query);
		if (rows == 0)	{
			g_sprintf (string, _("There is no genre \'%s\' in database, do you want "
						"to add it now?"), gtk_entry_get_text (GTK_ENTRY (tmp)));
			result = show_question_dlg (_("New genre"), string, 0);
			if (result == TRUE)	{
				g_sprintf (sql_query, "INSERT INTO generes (id, genere) VALUES (NULL, \'%s\');",
					gtk_entry_get_text (GTK_ENTRY (tmp)));
				sqlite3_prepare (database, sql_query, -1, &db_row, 0);
				status = sqlite3_step (db_row);
				g_printf (">>> Adding new genre, SQLite result: %d\n", status);
				sqlite3_finalize (db_row);
			}
		}
	}
	/* Sprawdzamy pole 'Publish house'	*/
	tmp = glade_xml_get_widget (xml, "publish_house_en");
	
	/*	Jezeli pole jest puste to je pomijamy	*/
	if (strcmp ( gtk_entry_get_text (GTK_ENTRY (tmp)), "") > 0)	{
		g_sprintf (sql_query, "SELECT COUNT(id) FROM publish_houses WHERE publish_house=\'%s\';",
					gtk_entry_get_text (GTK_ENTRY (tmp)));
		rows = count_returned_rows (sql_query);
		if (rows == 0)	{
			g_sprintf (string, _("There is no publish house \'%s\' in database, do you want "
						"to add it now?"), gtk_entry_get_text (GTK_ENTRY (tmp)));
			result = show_question_dlg (_("New publish house"), string, 0);
			if (result == TRUE)	{
				g_sprintf (sql_query, "INSERT INTO publish_houses (id, publish_house) VALUES (NULL, \'%s\');",
					gtk_entry_get_text (GTK_ENTRY (tmp)));
				sqlite3_prepare (database, sql_query, -1, &db_row, 0);
				status = sqlite3_step (db_row);
				g_printf (">>> Adding new publish house, SQLite result: %d\n", status);
				sqlite3_finalize (db_row);
			}
		}
	}
	
	/* Sprawdzamy pole 'Languages'	*/
	tmp = glade_xml_get_widget (xml, "language_en");
	
	/*	Jezeli pole jest puste to je pomijamy	*/
	if (strcmp ( gtk_entry_get_text (GTK_ENTRY (tmp)), "") > 0)	{
		g_sprintf (sql_query, "SELECT COUNT(id) FROM languages WHERE language=\'%s\';",
					gtk_entry_get_text (GTK_ENTRY (tmp)));
		rows = count_returned_rows (sql_query);
		if (rows == 0)	{
			g_sprintf (string, _("There is no language \'%s\' in database, do you want "
						"to add it now?"), gtk_entry_get_text (GTK_ENTRY (tmp)));
			result = show_question_dlg (_("New language"), string, 0);
			if (result == TRUE)	{
				g_sprintf (sql_query, "INSERT INTO languages (id, language) VALUES (NULL, \'%s\');",
					gtk_entry_get_text (GTK_ENTRY (tmp)));
				sqlite3_prepare (database, sql_query, -1, &db_row, 0);
				status = sqlite3_step (db_row);
				g_printf (">>> Adding new language, SQLite result: %d\n", status);
				sqlite3_finalize (db_row);
			}
		}
	}
	
	/* Sprawdzamy pole autorow	*/
	tmp = glade_xml_get_widget (xml, "m_first_author_en");

	/*	Jezeli pole jest puste to je pomijamy	*/
	if (strcmp ( gtk_entry_get_text (GTK_ENTRY (tmp)), "") > 0)	{
		g_sprintf (sql_query, "SELECT COUNT(id) FROM authors WHERE author=\'%s\';",
					gtk_entry_get_text (GTK_ENTRY (tmp)));
		rows = count_returned_rows (sql_query);
		if (rows == 0)	{
			g_sprintf (string, _("There is no author \'%s\' in database, do you want "
						"to add it now?"), gtk_entry_get_text (GTK_ENTRY (tmp)));
			result = show_question_dlg (_("New author"), string, 0);
			if (result == TRUE)	{
				g_sprintf (sql_query, "INSERT INTO authors (id, author) VALUES (NULL, \'%s\');",
					gtk_entry_get_text (GTK_ENTRY (tmp)));
				sqlite3_prepare (database, sql_query, -1, &db_row, 0);
				status = sqlite3_step (db_row);
				g_printf (">>> Adding new author, SQLite result: %d\n", status);
				sqlite3_finalize (db_row);
			}
		}
	}
	
	tmp = glade_xml_get_widget (xml, "second_author_en");
		
	/*	Jezeli pole jest puste to je pomijamy	*/
	if (strcmp ( gtk_entry_get_text (GTK_ENTRY (tmp)), "") > 0)	{
		g_sprintf (sql_query, "SELECT COUNT(id) FROM authors WHERE author=\'%s\';",
					gtk_entry_get_text (GTK_ENTRY (tmp)));
		rows = count_returned_rows (sql_query);
		if (rows == 0)	{
			g_sprintf (string, _("There is no author \'%s\' in database, do you want "
						"to add it now?"), gtk_entry_get_text (GTK_ENTRY (tmp)));
			result = show_question_dlg (_("New author"), string, 0);
			if (result == TRUE)	{
				g_sprintf (sql_query, "INSERT INTO authors (id, author) VALUES (NULL, \'%s\');",
					gtk_entry_get_text (GTK_ENTRY (tmp)));
				sqlite3_prepare (database, sql_query, -1, &db_row, 0);
				status = sqlite3_step (db_row);
				g_printf (">>> Adding new author, SQLite result: %d\n", status);
				sqlite3_finalize (db_row);
			}
		}
	}
	
	tmp = glade_xml_get_widget (xml, "third_author_en");
	
	/*	Jezeli pole jest puste to je pomijamy	*/
	if (strcmp ( gtk_entry_get_text (GTK_ENTRY (tmp)), "") > 0)	{
		g_sprintf (sql_query, "SELECT COUNT(id) FROM authors WHERE author=\'%s\';",
					gtk_entry_get_text (GTK_ENTRY (tmp)));
		rows = count_returned_rows (sql_query);
		if (rows == 0)	{
			g_sprintf (string, _("There is no author \'%s\' in database, do you want "
						"to add it now?"), gtk_entry_get_text (GTK_ENTRY (tmp)));
			result = show_question_dlg (_("New author"), string, 0);
			if (result == TRUE)	{
				g_sprintf (sql_query, "INSERT INTO authors (id, author) VALUES (NULL, \'%s\');",
					gtk_entry_get_text (GTK_ENTRY (tmp)));
				sqlite3_prepare (database, sql_query, -1, &db_row, 0);
				status = sqlite3_step (db_row);
				g_printf (">>> Adding new author, SQLite result: %d\n", status);
				sqlite3_finalize (db_row);
			}
		}
	}
	
	/* Sprawdzamy pole orginalnej wersji jezykowej	*/
	tmp = glade_xml_get_widget (xml, "orginal_language_en");
	
	/*	Jezeli pole jest puste to je pomijamy	*/
	if (strcmp ( gtk_entry_get_text (GTK_ENTRY (tmp)), "") > 0)	{
		g_sprintf (sql_query, "SELECT COUNT(id) FROM languages WHERE language=\'%s\';",
					gtk_entry_get_text (GTK_ENTRY (tmp)));
		rows = count_returned_rows (sql_query);
		if (rows == 0)	{
			g_sprintf (string, _("There is no language \'%s\' in database, do you want "
						"to add it now?"), gtk_entry_get_text (GTK_ENTRY (tmp)));
			result = show_question_dlg (_("New language"), string, 0);
			if (result == TRUE)	{
				g_sprintf (sql_query, "INSERT INTO languages (id, language) VALUES (NULL, \'%s\');",
					gtk_entry_get_text (GTK_ENTRY (tmp)));
				sqlite3_prepare (database, sql_query, -1, &db_row, 0);
				status = sqlite3_step (db_row);
				g_printf (">>> Adding new language, SQLite result: %d\n", status);
				sqlite3_finalize (db_row);
			}
		}
	}
	
	/* Sprawdzamy pola tlumaczy	*/
	tmp = glade_xml_get_widget (xml, "m_book_same_translators_chbox");
	/*	Sprawdzamy, czy ustawiono tego samego autora dla wszystkich ksiazek	*/
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
		tmp = glade_xml_get_widget (xml, "m_same_first_translator_en");
	
		/*	Jezeli pole jest puste to je pomijamy	*/
		if (strlen ( gtk_entry_get_text (GTK_ENTRY (tmp))) > 0)	{
			g_sprintf (sql_query, "SELECT COUNT(id) FROM translators WHERE translator=\'%s\';",
						gtk_entry_get_text (GTK_ENTRY (tmp)));
			rows = count_returned_rows (sql_query);
			if (rows == 0)	{
				g_sprintf (string, _("There is no tralnslator \'%s\' in database, do you want "
							"to add him now?"), gtk_entry_get_text (GTK_ENTRY (tmp)));
				result = show_question_dlg (_("New translator"), string, 0);
				if (result == TRUE)	{
					g_sprintf (sql_query, "INSERT INTO translators (id, translator) VALUES (NULL, \'%s\');",
						gtk_entry_get_text (GTK_ENTRY (tmp)));
					sqlite3_prepare (database, sql_query, -1, &db_row, 0);
					status = sqlite3_step (db_row);
					g_printf (">>> Adding new nranslator, SQLite result: %d\n", status);
					sqlite3_finalize (db_row);
				}
			}
		}
	
		tmp = glade_xml_get_widget (xml, "same_second_translator_en");
	
		/*	Jezeli pole jest puste to je pomijamy	*/
		if (strlen ( gtk_entry_get_text (GTK_ENTRY (tmp))) > 0)	{
			g_sprintf (sql_query, "SELECT COUNT(id) FROM translators WHERE translator=\'%s\';",
						gtk_entry_get_text (GTK_ENTRY (tmp)));
			rows = count_returned_rows (sql_query);
			if (rows == 0)	{
				g_sprintf (string, _("There is no translator \'%s\' in database, do you want "
							"to add him now?"), gtk_entry_get_text (GTK_ENTRY (tmp)));
				result = show_question_dlg (_("New translator"), string, 0);
				if (result == TRUE)	{
					g_sprintf (sql_query, "INSERT INTO translators (id, translator) VALUES (NULL, \'%s\');",
							gtk_entry_get_text (GTK_ENTRY (tmp)));
					sqlite3_prepare (database, sql_query, -1, &db_row, 0);
					status = sqlite3_step (db_row);
					g_printf (">>> Adding new translator, SQLite result: %d\n", status);
					sqlite3_finalize (db_row);
				}
			}
		}
		
		tmp = glade_xml_get_widget (xml, "same_third_translator_en");
	
		/*	Jezeli pole jest puste to je pomijamy	*/
		if (strlen ( gtk_entry_get_text (GTK_ENTRY (tmp))) > 0)	{
			g_sprintf (sql_query, "SELECT COUNT(id) FROM translators WHERE translator=\'%s\';",
						gtk_entry_get_text (GTK_ENTRY (tmp)));
			rows = count_returned_rows (sql_query);
			if (rows == 0)	{
				g_sprintf (string, _("There is no translator \'%s\' in database, do you want "
							"to add him now?"), gtk_entry_get_text (GTK_ENTRY (tmp)));
				result = show_question_dlg (_("New translator"), string, 0);
				if (result == TRUE)	{
					g_sprintf (sql_query, "INSERT INTO translators (id, translator) VALUES (NULL, \'%s\');",
						gtk_entry_get_text (GTK_ENTRY (tmp)));
					sqlite3_prepare (database, sql_query, -1, &db_row, 0);
					status = sqlite3_step (db_row);
					g_printf (">>> Adding new translator, SQLite result: %d\n", status);
					sqlite3_finalize (db_row);
				}
			}
		}
	}
}

/*
 *	Funkcja dodaje informacje o nowej ksiazce do bazy danych.
 *
 *	UWAGA! Funkcja NIE zapisuje wybranych ksiazek wchodzacych w skalad dodawanej
 *	ksiazki wielotomowej, tym zajmuje sie funkcja 'm_update_selected_books'
 */

void		m_insert_book		(MultiVolSharedDataStruc	*book_data)	{
	sqlite3_stmt		*db_row;
	gint				status;
	gchar				sql_query	[VERY_LONG_STRING];
	
	g_sprintf (sql_query, "INSERT INTO multi_volume_books_shared_data "
			 "(id, "
										 
			 "orginal_title, "
			 "title, "
										 
			 "first_author, "
			 "second_author, "
			 "third_author, "
			
			 "genere, "
			 "publish_house, "
										 
			 "language, "
			 "orginal_language, "
										 
			 "same_publish_year, "
			 "publish_year, "
			 
			 "same_translators, "
			 "first_translator, "
			 "second_translator, "
			 "third_translator, "
										 
			 "description) VALUES "
			 "(NULL, "
			 "\'%s\', "
			 "\'%s\', "
			 "%d, " 
			 "%d, " 
			 "%d, " 
			 "%d, "
			 "%d, "
			 "%d, "
			 "%d, "
			 "%d, "
			 "%d, "
			 "%d, "
			 "%d, "
			 "%d, "
			 "%d, "
			 "\'%s\');",		 
			 book_data->orginal_title,
			 book_data->title,
			 book_data->first_author,
			 book_data->second_author,
			 book_data->third_author,
			 book_data->genere,
			 book_data->publish_house,
			 book_data->language,
			 book_data->orginal_language,
			 book_data->same_publish_year,
			 book_data->publish_year,
			 book_data->same_translators,
			 book_data->first_translator,
			 book_data->second_translator,
			 book_data->third_translator,
			 book_data->description);
			 
	sqlite3_prepare (database, sql_query, -1, &db_row, 0);
	status = sqlite3_step (db_row);
	g_printf ("\t<<< Inserting book into database, SQLite result: %d\n", status);
	sqlite3_finalize (db_row);
	
	g_free (book_data);
}

/*
 *	Funkcja wyciaga z bazy danych informacje o ksiazce o wskazanym ID
 */

MultiVolSharedDataStruc		*m_select_book_data	(gint	*id)	{
	MultiVolSharedDataStruc	*book_data;
	sqlite3_stmt			*db_row;
	gchar					sql_query	[LONG_STRING];
	gint					status;
	
	book_data = g_new (MultiVolSharedDataStruc, 1);
	
	g_sprintf (sql_query, "SELECT "
			 "id, "
										 
			 "orginal_title, "
			 "title, "
										 
			 "first_author, "
			 "second_author, "
			 "third_author, "
			
			 "genere, "
			 "publish_house, "
										 
			 "language, "
			 "orginal_language, "
										 
			 "same_publish_year, "
			 "publish_year, "
			 
			 "same_translators, "
			 "first_translator, "
			 "second_translator, "
			 "third_translator, "
										 
			 "description "
			 "FROM multi_volume_books_shared_data WHERE id=%d;", *id);
			 
	sqlite3_prepare (database, sql_query, -1, &db_row, 0);
	status = sqlite3_step (db_row);
	g_printf ("\t<<< Selecting book\'s informations from database, SQLite result: %d\n", status);
	
	book_data->id = sqlite3_column_int (db_row, 0);
	g_sprintf (book_data->orginal_title, "%s", sqlite3_column_text (db_row, 1));
	g_sprintf (book_data->title, "%s", sqlite3_column_text (db_row, 2));
	book_data->first_author = sqlite3_column_int (db_row, 3);
	book_data->second_author = sqlite3_column_int (db_row, 4);
	book_data->third_author = sqlite3_column_int (db_row, 5);
	book_data->genere = sqlite3_column_int (db_row, 6);
	book_data->publish_house = sqlite3_column_int (db_row, 7);
	book_data->language = sqlite3_column_int (db_row, 8);
	book_data->orginal_language = sqlite3_column_int (db_row, 9);
	book_data->same_publish_year = sqlite3_column_int (db_row, 10);
	book_data->publish_year = sqlite3_column_int (db_row, 11);
	book_data->same_translators = sqlite3_column_int (db_row, 12);
	book_data->first_translator = sqlite3_column_int (db_row, 13);
	book_data->second_translator = sqlite3_column_int (db_row, 14);
	book_data->third_translator = sqlite3_column_int (db_row, 15);
	g_sprintf (book_data->description, "%s", sqlite3_column_text (db_row, 16));
	
	sqlite3_finalize (db_row);	 
	
	/*	Wyciafamy jeszcze liczbe tomow wchodzacych w skald tej ksiazki 
	 *	wielotomowej	*/
	g_sprintf (sql_query, "SELECT COUNT(id) FROM multi_volume_books_private_data "
		"WHERE multi_id=%d;", book_data->id);
	sqlite3_prepare (database, sql_query, -1, &db_row, 0);
	status = sqlite3_step (db_row);
	g_printf ("\t<<< Selecting number of volumes, SQLite result: %d\n", status);
	book_data->volumes_number = sqlite3_column_int (db_row, 0);
	sqlite3_finalize (db_row);	
	
	return book_data;
}

/*
 *	Funkcja wyswietla informacje o wybranej ksiazce w oknie 'main_win'
 *	Przy okazji funkcja dokonuje malej obrobki wyswietlanych danych, wiecej
 *	informacji na ten temat w kodzie funkcji.
 */

void		m_display_book_data		(MultiVolSharedDataStruc	*book_data)	{
	GladeXML			*xml;
	GtkTreeModel		*store;
	GtkTreeIter			iter;
	GtkTextBuffer		*text_buffer;
	GtkWidget			*tmp;
	gint				status;
	gint				rows;
	gchar				sql_query	[SHORT_STRING];
	gchar				string		[SHORT_STRING];
	gchar				item_name	[SHORT_STRING];
	sqlite3_stmt		*db_row;
	
	xml = glade_get_widget_tree (main_win);
	g_printf ("<<< Displaying book's informations...\n");
	
	set_widget_sensinitive (main_win, "main_multi_book_vbox", FALSE);
	
	/*	Wyswietlamy ID ksiazki	*/
	tmp = glade_xml_get_widget (xml, "m_book_id_lb");
	g_sprintf (string, "%d", book_data->id);
	gtk_label_set_text (GTK_LABEL (tmp), string);
	
	tmp = glade_xml_get_widget (xml, "m_title_lb");
	gtk_label_set_text (GTK_LABEL (tmp), book_data->title);
	
	/*	Wyszukujemy autorow po numerach id zwroconych w strukturze 'book_data'	*/
	
	tmp = glade_xml_get_widget (xml, "m_author_lb");
	/*	Jezeli wszyscy autorzy sa ustawieni na 0, oznacza to, ze nie podano
		zadnego autora	*/
	if (book_data->first_author == 0 &&
		book_data->second_author == 0 &&
		book_data->third_author == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (book_data->first_author > 0)	{		
		get_author_name (item_name, book_data->first_author);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
		
		if (book_data->second_author > 0)	{
			get_author_name (item_name, book_data->second_author);
			g_sprintf (string, "%s\n%s", gtk_label_get_text (GTK_LABEL (tmp)),
						item_name);
			gtk_label_set_text (GTK_LABEL (tmp), string);
		}
		if (book_data->third_author > 0)	{
			get_author_name (item_name, book_data->third_author);
			g_sprintf (string, "%s\n%s", gtk_label_get_text (GTK_LABEL (tmp)),
						item_name);
			gtk_label_set_text (GTK_LABEL (tmp), string);
		}
	}
	else	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	
	/*	Wyciagamy gatunek z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "m_genere_lb");
	/*	Jezeli gatunek ustawiony jest na 0, oznacza to, ze nie podano
		zadnego gatunku	*/
	if (book_data->genere == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (book_data->genere > 0)	{
		get_genere_name (item_name, book_data->genere);		
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
	}
	
	/*	Wyciagamy wydawnictwo z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "m_publish_house_lb");
	/*	Jezeli wydawnictwo ustawione jest na 0, oznacza to, ze nie podano
		zadnego wydawnictwa	*/
	if (book_data->publish_house == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (book_data->publish_house > 0)	{
		get_publish_house_name (item_name, book_data->publish_house);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
	}
	
	/*	Wyciagamy jezyk z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "m_language_lb");
	/*	Jezeli jezyk  ustawiony jest na 0, oznacza to, ze nie podano
		zadnego jezyka	*/
	if (book_data->language == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (book_data->language > 0)	{
		get_language_name (item_name, book_data->language);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
	}
	
	/*	Wyswietlamy licze tomow	*/
	tmp = glade_xml_get_widget (xml, "m_volumes_number_lb");
	g_sprintf (string, "%d", book_data->volumes_number);
	gtk_label_set_text (GTK_LABEL (tmp), string);
	
	/*	Wyswietlamy orginalny tytul ksiazki	*/
	tmp = glade_xml_get_widget (xml, "m_orginal_title_lb");
	if (strlen (book_data->orginal_title) == 0)	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else	{
		gtk_label_set_text (GTK_LABEL (tmp), book_data->orginal_title);
	}
	
	/*	Wyciagamy orginalny jezyk z bazy danych	*/
	tmp = glade_xml_get_widget (xml, "m_orginal_language_lb");
	/*	Jezeli orginalny jezyk  ustawiony jest na 0, oznacza to, ze nie podano
		zadnego jezyka	*/
	if (book_data->orginal_language == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (book_data->orginal_language > 0)	{
		get_language_name (item_name, book_data->orginal_language);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
	}
	
	tmp = glade_xml_get_widget (xml, "m_publish_year_lb");
	if (book_data->same_publish_year == 1)	{
		g_sprintf (string, "%d", book_data->publish_year);
		gtk_label_set_text (GTK_LABEL (tmp), string);
	}
	else {
		if (book_data->publish_year == 0)	{
			gtk_label_set_text (GTK_LABEL (tmp), _("Not concern"));
		}
	}
	
	/*	Wyszukujemy tlumaczy po numerach id zwroconych w strukturze 'book_data'	
	 *	tylko gdy sa oni wspolni dla wszystkich tomow.	*/
	tmp = glade_xml_get_widget (xml, "m_translator_lb");
	if (book_data->same_translators == 1)	{
		/*	Jezeli wszyscy tlumacze sa ustawieni na 0, oznacza to, ze nie podano
			zadnego tlimacza	*/
		if (book_data->first_translator == 0 &&
			book_data->second_translator == 0 &&
			book_data->third_translator == 0)
		{
			gtk_label_set_text (GTK_LABEL (tmp), _("Not concern"));
		}
		else if (book_data->first_translator > 0)	{		
			get_translator_name (item_name, book_data->first_translator);
			gtk_label_set_text (GTK_LABEL (tmp), item_name);

			if (book_data->second_translator > 0)	{
				get_translator_name (item_name, book_data->second_translator);
				g_sprintf (string, "%s\n%s", gtk_label_get_text (GTK_LABEL (tmp)),
							item_name);
				gtk_label_set_text (GTK_LABEL (tmp), string);
			}
			if (book_data->third_translator > 0)	{
				get_translator_name (item_name, book_data->third_translator);
				g_sprintf (string, "%s\n%s", gtk_label_get_text (GTK_LABEL (tmp)),
							item_name);
				gtk_label_set_text (GTK_LABEL (tmp), string);
			}
		}
	}
	else	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Not concern"));
	}
	
	tmp = glade_xml_get_widget (xml, "m_books_list");
	store = gtk_tree_view_get_model (GTK_TREE_VIEW (tmp));
	
	g_printf("\t>>> Loading selected books titles, counting books...\n");
	
	g_sprintf (sql_query, "SELECT COUNT(id) FROM multi_volume_books_private_data WHERE multi_id=%d;", 
			book_data->id);
	rows = count_returned_rows (sql_query);
	g_printf ("<<< Selected books in database: %d\n", rows);
	g_sprintf (sql_query, "SELECT id, title FROM multi_volume_books_private_data WHERE multi_id=%d;", 
			book_data->id);
	sqlite3_prepare (database, sql_query, -1, &db_row, 0);	
	status = sqlite3_step (db_row);
	if (rows > 0)	{
		while (status == 100)		{
			gtk_list_store_append (GTK_LIST_STORE (store), &iter);
			gtk_list_store_set (GTK_LIST_STORE (store), &iter,
							0, sqlite3_column_int (db_row, 0), 
							1, sqlite3_column_text (db_row, 1),
							-1);
			status = sqlite3_step (db_row);
			}
	}
	sqlite3_finalize (db_row);
	
	tmp = glade_xml_get_widget (xml, "m_description_text");
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tmp));
	gtk_text_buffer_set_text (text_buffer, book_data->description, strlen (book_data->description));
	
	set_widget_sensinitive (main_win, "main_multi_book_vbox", TRUE);
	g_free (book_data);
}

/*
 *	Funkcja wyswietla informacje o ksiazce w oknie 'property_win'
 */

void		m_show_book_data	(GtkWidget	*widget, gint	*id)	{
	GladeXML				*xml;
	GtkTreeModel			*store;
	GtkTreeIter				iter;
	GtkTextBuffer			*text_buffer;
	GtkWidget				*tmp;
	gint					status;
	gint					rows;
	gchar					sql_query	[SHORT_STRING];
	gchar					string		[SHORT_STRING];
	gchar					item_name	[SHORT_STRING];
	sqlite3_stmt			*db_row;
	MultiVolSharedDataStruc	*book_data;
	
	xml = glade_get_widget_tree (widget);
	book_data = m_select_book_data (id);
	
	g_printf ("<<< Displaying book's informations...\n");
	
	tmp = glade_xml_get_widget (xml, "title_lb");
	gtk_label_set_text (GTK_LABEL (tmp), book_data->title);
	
	/*	Wyszukujemy autorow po numerach id zwroconych w strukturze 'book_data'	*/
	
	tmp = glade_xml_get_widget (xml, "author_lb");
	/*	Jezeli wszyscy autorzy sa ustawieni na 0, oznacza to, ze nie podano
		zadnego autora	*/
	if (book_data->first_author == 0 &&
		book_data->second_author == 0 &&
		book_data->third_author == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (book_data->first_author > 0)	{		
		get_author_name (item_name, book_data->first_author);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
		
		if (book_data->second_author > 0)	{
			get_author_name (item_name, book_data->second_author);
			g_sprintf (string, "%s\n%s", gtk_label_get_text (GTK_LABEL (tmp)),
						item_name);
			gtk_label_set_text (GTK_LABEL (tmp), string);
		}
		if (book_data->third_author > 0)	{
			get_author_name (item_name, book_data->third_author);
			g_sprintf (string, "%s\n%s", gtk_label_get_text (GTK_LABEL (tmp)),
						item_name);
			gtk_label_set_text (GTK_LABEL (tmp), string);
		}
	}
	else	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	
	/*	Wyciagamy gatunek z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "genere_lb");
	/*	Jezeli gatunek ustawiony jest na 0, oznacza to, ze nie podano
		zadnego gatunku	*/
	if (book_data->genere == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (book_data->genere > 0)	{
		g_sprintf (sql_query, "SELECT genere FROM generes WHERE id=%d;",
					book_data->genere);
		sqlite3_prepare (database, sql_query, -1, &db_row, 0);
		status = sqlite3_step (db_row);
		g_printf ("\t<<< Selecting genre from database, SQLite result: %d\n", status);	
		
		gtk_label_set_text (GTK_LABEL (tmp), (gpointer) sqlite3_column_text (db_row, 0));
		sqlite3_finalize (db_row);
	}
	
	/*	Wyciagamy wydawnictwo z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "publish_house_lb");
	/*	Jezeli wydawnictwo ustawione jest na 0, oznacza to, ze nie podano
		zadnego wydawnictwa	*/
	if (book_data->publish_house == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Not concern"));
	}
	else if (book_data->publish_house > 0)	{
		get_publish_house_name (item_name, book_data->publish_house);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
	}
	
	/*	Wyciagamy jezyk z bazy danych	*/
	tmp = glade_xml_get_widget (xml, "language_lb");
	/*	Jezeli jezyk  ustawiony jest na 0, oznacza to, ze nie podano
		zadnego jezyka	*/
	if (book_data->language == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (book_data->language > 0)	{
		get_language_name (item_name, book_data->language);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
	}
	
	tmp = glade_xml_get_widget (xml, "orginal_title_lb");
	if (strlen (book_data->orginal_title) == 0)	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else	{
		gtk_label_set_text (GTK_LABEL (tmp), book_data->orginal_title);
	}
	
	/*	Wyciagamy orginalny jezyk z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "orginal_language_lb");
	/*	Jezeli orginalny jezyk  ustawiony jest na 0, oznacza to, ze nie podano
		zadnego jezyka	*/
	if (book_data->orginal_language == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (book_data->orginal_language > 0)	{
		get_language_name (item_name, book_data->orginal_language);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
	}
	
	tmp = glade_xml_get_widget (xml, "isbn_lb");
	gtk_label_set_text (GTK_LABEL (tmp), _("Not concern"));

	
	tmp = glade_xml_get_widget (xml, "publish_year_lb");
	if (book_data->publish_year == 0)	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else {
		g_sprintf (string, "%d", book_data->publish_year);
		gtk_label_set_text (GTK_LABEL (tmp), string);
	}
	
	tmp = glade_xml_get_widget (xml, "pages_number_lb");
	gtk_label_set_text (GTK_LABEL (tmp), _("Not concern"));
	
	/*	Wyszukujemy tlumaczy po numerach id zwroconych w strukturze 'book_data'	*/
	
	tmp = glade_xml_get_widget (xml, "translator_lb");
	/*	Jezeli wszyscy tlumacze sa ustawieni na 0, oznacza to, ze nie podano
		zadnego tlimacza	*/
	if (book_data->first_translator == 0 &&
		book_data->second_translator == 0 &&
		book_data->third_translator == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Not concern"));
	}
	else if (book_data->first_translator > 0)	{
		g_sprintf (sql_query, "SELECT translator FROM translators WHERE id=%d;",
					book_data->first_translator);
		sqlite3_prepare (database, sql_query, -1, &db_row, 0);
		status = sqlite3_step (db_row);
		g_printf ("\t<<< Selecting first translator from database, SQLite result: %d\n", status);	
		
		gtk_label_set_text (GTK_LABEL (tmp), (gpointer) sqlite3_column_text (db_row, 0));
		sqlite3_finalize (db_row);
		
		if (book_data->second_translator > 0)	{
			g_sprintf (sql_query, "SELECT translator FROM translators WHERE id=%d;",
					book_data->second_translator);
			sqlite3_prepare (database, sql_query, -1, &db_row, 0);
			status = sqlite3_step (db_row);
			g_printf ("\t<<< Selecting second translator from database, SQLite result: %d\n", status);	
			
			g_sprintf (string, "%s\n%s", gtk_label_get_text (GTK_LABEL (tmp)),
						sqlite3_column_text (db_row, 0));
			gtk_label_set_text (GTK_LABEL (tmp), string);
			sqlite3_finalize (db_row);
		}
		if (book_data->third_translator > 0)	{
			g_sprintf (sql_query, "SELECT translator FROM translators WHERE id=%d;",
					book_data->third_translator);
			sqlite3_prepare (database, sql_query, -1, &db_row, 0);
			status = sqlite3_step (db_row);
			g_printf ("\t<<< Selecting third translator from database, SQLite result: %d\n", status);	
			
			g_sprintf (string, "%s\n%s", gtk_label_get_text (GTK_LABEL (tmp)),
						sqlite3_column_text (db_row, 0));
			gtk_label_set_text (GTK_LABEL (tmp), string);
			sqlite3_finalize (db_row);
		}
	}
	else	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	
	tmp = glade_xml_get_widget (xml, "description_text");
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tmp));
	gtk_text_buffer_set_text (text_buffer, book_data->description, strlen (book_data->description));
	
	tmp = glade_xml_get_widget (xml, "books_list");
	store = gtk_tree_view_get_model (GTK_TREE_VIEW (tmp));
	
	/*	Ladujemy tytylu poszczegolnych tomow do listy	*/
	g_printf("\t>>> Loading volume's titles, counting books...\n");
	
	g_sprintf (sql_query, "SELECT COUNT(id) FROM multi_volume_books_private_data WHERE multi_id=%d;", 
			book_data->id);
	rows = count_returned_rows (sql_query);
	g_printf ("<<< Selected books in database: %d\n", rows);
	g_sprintf (sql_query, "SELECT id, title FROM multi_volume_books_private_data WHERE multi_id=%d;", 
			book_data->id);
	sqlite3_prepare (database, sql_query, -1, &db_row, 0);	
	status = sqlite3_step (db_row);
	if (rows > 0)	{
		while (status == 100)		{
			gtk_list_store_append (GTK_LIST_STORE (store), &iter);
			gtk_list_store_set (GTK_LIST_STORE (store), &iter,
							0, sqlite3_column_int (db_row, 0), 
							1, sqlite3_column_text (db_row, 1),
							-1);
			status = sqlite3_step (db_row);
			}
	}
	sqlite3_finalize (db_row);
	/*	Pokazujemy zakladke z lista	*/
	tmp = glade_xml_get_widget (xml, "scrolledwindow10");
	gtk_widget_show (tmp);

	set_widget_sensinitive (tmp, "property_notebook", TRUE);
	
	g_free (book_data);
}

/*
 *	Funkcja usuwa jednotomowa ksiazke o wskazanym id oraz wszyhstkie tomy 
 *	wchodzace w jej sklad.
 */

void		m_remove_book		(gint	*id)	{
	gint				status;
	gchar				sql_query	[SHORT_STRING];
	sqlite3_stmt		*db_row;
	
	g_sprintf (sql_query, "DELETE FROM multi_volume_books_shared_data WHERE id=%d;", *id);
	sqlite3_prepare (database, sql_query, -1, &db_row, 0);
	status = sqlite3_step (db_row);
	g_printf ("<<< Removing multi volume book from database step 1/2, SQLite result: %d\n", status);
	sqlite3_finalize (db_row);
	
	g_sprintf (sql_query, "DELETE FROM multi_volume_books_private_data WHERE multi_id=%d;", *id);
	sqlite3_prepare (database, sql_query, -1, &db_row, 0);
	status = sqlite3_step (db_row);
	g_printf ("<<< Removing multi volume book from database, step 2/2, SQLite result: %d\n", status);
	sqlite3_finalize (db_row);
}

/*
 *	Funkcja czysci wszytskie pola wyswieltajace informacje o ksiazkach
 *	jednotomowych w oknie 'main_win' i dezaktywuje przyciski odpowiedzialne
 *	za edycje, usuwanie itp. ksiazki jednotomowej o oknie 'main_wion'.
 *	Funkcja jest przydatna np. gdy usuniemy ksiazke z bazy danych.
 */

void				m_clear_main_win		(void)	{
	GladeXML		*xml;
	GtkTextBuffer	*text_buffer;
	GtkWidget		*tmp;
	
	xml = glade_get_widget_tree (main_win);
	tmp = glade_xml_get_widget (xml, "m_book_id_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_title_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_author_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_genere_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_publish_house_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_language_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_volumes_number_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_orginal_title_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_orginal_language_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_publish_year_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_translator_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_books_list");
	clear_list_list (tmp);
	
	tmp = glade_xml_get_widget (xml, "m_description_text");
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tmp));
	gtk_text_buffer_set_text (text_buffer, "", strlen (""));
	
	set_widget_sensinitive (tmp, "m_remove_but", FALSE);
	set_widget_sensinitive (tmp, "m_edit_but", FALSE);
}

/*
 *	Funkcja zapisuje zmodyfikowane informacje o ksiazce do bazy danych
 */

void	m_update_book	(MultiVolSharedDataStruc	*book_data)	{
	sqlite3_stmt		*db_row;
	gint				status;
	gchar				sql_query	[VERY_LONG_STRING];
	
	g_sprintf (sql_query, "UPDATE multi_volume_books_shared_data SET "										  
		 "orginal_title=\'%s\', "
		 "title=\'%s\', "
									 
		 "first_author=%d, "
		 "second_author=%d, "
		 "third_author=%d, "
		
		 "genere=%d, "
		 "publish_house=%d, "
									 
		 "language=%d, "
		 "orginal_language=%d, "
									 
		 "same_publish_year=%d, "
		 "publish_year=%d, "
		 
		 "same_translators=%d, "
		 "first_translator=%d, "
		 "second_translator=%d, "
		 "third_translator=%d, "
									 
		 "description=\'%s\' "
		 
		 "WHERE "
		 "id=%d;",
		 book_data->orginal_title,
		 book_data->title,
		 book_data->first_author,
		 book_data->second_author,
		 book_data->third_author,
		 book_data->genere,
		 book_data->publish_house,
		 book_data->language,
		 book_data->orginal_language,
		 book_data->same_publish_year,
		 book_data->publish_year,
		 book_data->same_translators,
		 book_data->first_translator,
		 book_data->second_translator,
		 book_data->third_translator,
		 book_data->description,
		 book_data->id);
		
	sqlite3_prepare (database, sql_query, -1, &db_row, 0);
	status = sqlite3_step (db_row);
	
	g_printf ("<<< Updating book's informations from database, SQLite result: %d\n", status);
	
	sqlite3_finalize (db_row);
	g_free (book_data);
}

/*
 *	Funkcja laduje informacje o ksiazce o wskazanym id do okna 'm_book_win'
 *	i przygotowuje dane na temat ksiazki do edycji.
 */

void		m_load_edit_win			(MultiVolSharedDataStruc	*book_data,
									 GtkWidget					*widget)	{
	GladeXML				*xml;
	GtkTextBuffer			*text_buffer;
	GtkTreeModel			*store;
	GtkTreeIter 			iter;		
	GtkWidget				*tmp;
	gint					status;
	gint					rows;
	gchar					sql_query	[SHORT_STRING];
	gchar					string		[VERY_SHORT_STRING];
	gchar					item_name	[SHORT_STRING];	
	sqlite3_stmt			*db_row;
	
	xml = glade_get_widget_tree (widget);									 
										 
	tmp = glade_xml_get_widget (xml, "book_id_lb");
	g_sprintf (string, "%d", book_data->id);
	gtk_label_set_text (GTK_LABEL (tmp), string);
	
	tmp = glade_xml_get_widget (xml, "title_en");
	gtk_entry_set_text (GTK_ENTRY (tmp), book_data->title);
	
	/*	Wyciagamy gatunek z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "genere_en");
	/*	Jezeli gatunek ustawiony jest na 0, oznacza to, ze nie podano
		zadnego gatunku	*/
	if (book_data->genere == 0)
	{
		gtk_entry_set_text (GTK_ENTRY (tmp), "");
	}
	else if (book_data->genere > 0)	{
		get_genere_name (item_name, book_data->genere);		
		gtk_entry_set_text (GTK_ENTRY (tmp), item_name);
	}
	
	/*	Wyciagamy wydawnictwo z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "publish_house_en");
	/*	Jezeli wydawnictwo ustawione jest na 0, oznacza to, ze nie podano
		zadnego wydawnictwa	*/
	if (book_data->publish_house == 0)
	{
		gtk_entry_set_text (GTK_ENTRY (tmp), "");
	}
	else if (book_data->publish_house > 0)	{
		get_publish_house_name (item_name, book_data->publish_house);
		gtk_entry_set_text (GTK_ENTRY (tmp), item_name);
	}
	
	/*	Wyciagamy jezyk z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "language_en");
	/*	Jezeli jesyk ustawiony jest na 0, oznacza to, ze nie podano
		zadnego jezyka	*/
	if (book_data->language == 0)
	{
		gtk_entry_set_text (GTK_ENTRY (tmp), "");
	}
	else if (book_data->language > 0)	{
		get_language_name (item_name, book_data->language);
		gtk_entry_set_text (GTK_ENTRY (tmp), item_name);
	}
	
	/*	Wyciagamy autorow z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "m_first_author_en");
	/*	Jezeli pierwszy autor ustawiony jest na 0, oznacza to, ze nie podano
		zadnego gatunku, a tym samym nie logiczne jest, ze podano
		dodatkowych autorowm dlatego pole rozszerzajace "additional_authors_exp"
		zostaje dezaktywowane, jezeli natomias pole pierwszego autora ma wartosci
		wieksza niz 0, pole 'additional_authors_exp' zostanie aktywowane, a 
		dodatkowi autorzy zostana zaladowani do okna	*/
	if (book_data->first_author == 0)
	{
		gtk_entry_set_text (GTK_ENTRY (tmp), "");
		set_widget_sensinitive (tmp, "additional_authors_exp", FALSE);
	}
	else if (book_data->first_author > 0)	{
		get_author_name (item_name, book_data->first_author);
		gtk_entry_set_text (GTK_ENTRY (tmp), item_name);
		
		set_widget_sensinitive (tmp, "additional_authors_exp", TRUE);
		
		/*	Teraz sprawdzamy, czy nastepni autorzy sa podani, jezeli tak, to 
			ladujemy ich do okna i zaznaczamy odpowiednie 'patszki'	*/
		if (book_data->second_author > 0)	{
			tmp = glade_xml_get_widget (xml, "second_author_en");
			get_author_name (item_name, book_data->second_author);
			gtk_entry_set_text (GTK_ENTRY (tmp), item_name);
		}
		
		if (book_data->third_author > 0)	{
			tmp = glade_xml_get_widget (xml, "third_author_en");
			get_author_name (item_name, book_data->third_author);
			gtk_entry_set_text (GTK_ENTRY (tmp), item_name);
		}
	}

	/*	Wyciagamy orginalny tytul ksiazki	*/
	tmp = glade_xml_get_widget (xml, "orginal_title_en");
	gtk_entry_set_text (GTK_ENTRY (tmp), book_data->orginal_title);
	
	/*	Wyciagamy jezyk z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "orginal_language_en");
	/*	Jezeli jesyk ustawiony jest na 0, oznacza to, ze nie podano
		zadnego jezyka	*/
	if (book_data->orginal_language == 0)
	{
		gtk_entry_set_text (GTK_ENTRY (tmp), "");
	}
	else if (book_data->orginal_language > 0)	{
		get_language_name (item_name, book_data->orginal_language);
		gtk_entry_set_text (GTK_ENTRY (tmp), item_name);
	}
	if (book_data->same_publish_year == 1)	{
		tmp = glade_xml_get_widget (xml, "same_publish_year_spin");
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (tmp), book_data->publish_year);
		tmp = glade_xml_get_widget (xml, "m_book_same_publish_year_check");
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp), TRUE);
		set_widget_sensinitive (tmp, "same_publish_year_spin", TRUE);
	}
	
	/*	Wyciagamy tlumaczy z bazy danych	*/
	tmp = glade_xml_get_widget (xml, "m_same_first_translator_en");
	/*	Patrz ladowanie autorow w tej funkcji	*/
	if (book_data->first_translator == 0)
	{
		gtk_entry_set_text (GTK_ENTRY (tmp), "");
		set_widget_sensinitive (tmp, "same_additional_translators_exp", FALSE);
	}
	else if (book_data->first_translator > 0)	{
		get_translator_name (item_name, book_data->first_translator);
		gtk_entry_set_text (GTK_ENTRY (tmp), item_name);
		
		tmp = glade_xml_get_widget (xml, "m_book_same_translators_chbox");
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp), TRUE);
		
		set_widget_sensinitive (tmp, "same_additional_translators_exp", TRUE);

		if (book_data->second_translator > 0)	{
			tmp = glade_xml_get_widget (xml, "same_second_translator_en");
			get_translator_name (item_name, book_data->second_translator);
			gtk_entry_set_text (GTK_ENTRY (tmp), item_name);
		}
		
		if (book_data->third_translator > 0)	{
			tmp = glade_xml_get_widget (xml, "same_third_translator_en");
			get_translator_name (item_name, book_data->third_translator);
			gtk_entry_set_text (GTK_ENTRY (tmp), item_name);
		}
	}
	
	tmp = glade_xml_get_widget (xml, "description_text");
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tmp));
	gtk_text_buffer_set_text (text_buffer, book_data->description, strlen (book_data->description));
	
	g_free (book_data);
}

/*
 *	Funkcja ustawia wartosc pol 'same_publish_year' i 'same_translator' we wszytskich
 *	strukturach zawierajacych dane o tomach.
 *	Wartosci argumentow:
 *	- widget - wskazuje na okno 'm_book_win'
 *	- data_type:
 *		- 0 - ustawiamy wspolna date wydania
 *		- 1 - ustawiamy wspolnych tlumaczy
 *	- mode - wartosc 0 lub 1
 */

void		m_set_same_data_status			(GtkWidget	*widget,
											 gint		data_type,
											 gint		mode)
{
	MultiVolEditPrivDataStruc	*volume_data;
	GladeXML					*xml;
	GtkTreeModel				*model;
  	GtkTreeIter					iter;
	GtkWidget					*tmp;
	gint						adress;
	gboolean					result;

	xml = glade_get_widget_tree (widget);
	
	tmp = glade_xml_get_widget (xml, "books_list");
	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tmp));

	result = gtk_tree_model_get_iter_first (model, &iter);
	while (result == TRUE)	{		
		/*	Pobieramy adres do struktury	*/
		gtk_tree_model_get (model, &iter,
			2, &adress,
			-1);
		volume_data = (MultiVolEditPrivDataStruc *) adress;
		
		/*	Sprawdzamy typ edytowanych danych	*/
		switch (data_type)	{
			case 0:
				/*	Ustawiamy wartosc dla wspolengo roku wydania	*/
				volume_data->same_publish_year = mode;
				break;
			case 1:
				/*	Ustalamy wartos dla wspolych tlumaczy	*/
				volume_data->same_translators = mode;
				break;
		}

		/*	Pobieramy nastepny wiersz	*/
		result = gtk_tree_model_iter_next (model, &iter);
	}									 
}

/*
 *	Funkcja kontroluje ilosc otwartych elementow 'GtkExpander' woknie 'main_win',
 *	w zakladce wyswieltajacej informacje o ksiazce wielotomowej. W sytuacji, gdy
 *	otwarte sa wszystkie elemnty rozszerzajace wysokosc okna osiaga 
 *	wartosc ponad 1200 pikseli, co moze uniemozliwic uzywanie okna przy 
 *	mniejszych rozdzielczosciach ekranu. Ta funkcja kontroluje ilosc otwartych
 *	elementow rozszerzajacych w tym samym czasie, ograniczajac ja do 1
 *	otwartego elementu 'GtkExpander'.
 *
 *	Argument 'mode' oznacza, ze otwarty zostal element:
 *	- 'm_books_list_exp'
 *	- 'm_additional_informations_exp'
 *	- 'm_description_exp'
 */

void		m_control_main_win_expanders	(gint	mode)	{
	GladeXML	*xml;
	GtkWidget	*m_books_list_exp;
	GtkWidget	*m_additional_informations_exp;
	GtkWidget	*m_description_exp;
	
	xml = glade_get_widget_tree (main_win);
	m_books_list_exp = glade_xml_get_widget (xml, "m_books_list_exp");
	m_additional_informations_exp = glade_xml_get_widget (xml,
				"m_additional_informations_exp");
	m_description_exp = glade_xml_get_widget (xml, "m_description_exp");
	
	switch (mode)	{
		case 0:
			gtk_expander_set_expanded (GTK_EXPANDER (m_additional_informations_exp), FALSE);
			gtk_expander_set_expanded (GTK_EXPANDER (m_description_exp), FALSE);
			break;
		case 1:
			gtk_expander_set_expanded (GTK_EXPANDER (m_books_list_exp), FALSE);
			gtk_expander_set_expanded (GTK_EXPANDER (m_description_exp), FALSE);
			break;
		case 2:
			gtk_expander_set_expanded (GTK_EXPANDER (m_books_list_exp), FALSE);
			gtk_expander_set_expanded (GTK_EXPANDER (m_additional_informations_exp), FALSE);
			break;
	}
}
