/***************************************************************************
 *            m_volume-fun.c
 *
 *  Fri Dec  2 17:34:29 2005
 *  Copyright  2005  Tomek Jankowski
 *  Email	tomcio-j@o2.pl
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glade/glade.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <glib/gi18n-lib.h>
#include <stdio.h>
#include <string.h>

#include "m_volume-fun.h"
#include "book-fun.h"
#include "database-fun.h"
#include "interface-fun.h"

#define VERY_SHORT_STRING	12
#define SHORT_STRING		128
#define MEDIUM_STRING		512
#define	LONG_STRING			1024
#define	DESCRIPTION_STRING	2048
#define	VERY_LONG_STRING	6144

/*
 *	Funkcja laduje informacje o poszczegolnych tomach tomach do okna edycji
 *	'm_book_win'. Konkretnie funkcja operuje na liscie 'books_list. wstawiajac
 *	do niej numer id tomu oraz adres do struktury, ktora przechowuje dane o
 *	danym tomie.
 */
 
void	m_volume_load_volumes	(GtkWidget	*widget,
								 gint		m_book_id)
{
	GladeXML					*xml;
	GtkTreeModel				*store;
	GtkTreeIter 				iter;
	GtkWidget					*tmp;
	gint						first_status;
	gint						second_status;
	gint						rows;
	gchar						sql_query	[MEDIUM_STRING];
	gchar						item_name	[MEDIUM_STRING];
	sqlite3_stmt				*first_db_row;
	sqlite3_stmt				*second_db_row;
	
	xml = glade_get_widget_tree (widget);
	tmp = glade_xml_get_widget (xml, "books_list");
	
	store = gtk_tree_view_get_model (GTK_TREE_VIEW (tmp));
	
	g_printf ("\t<<< Selecting volumes data...\n");	
	/*	Pierwsze zapytanie	*/
	g_sprintf (sql_query, "SELECT "
		"id, "
		"multi_id, "
		"title, "
		"orginal_title, "
		"isbn, "

		"publish_year, "
		
		"pages, "
		
		"first_translator, "
		"second_translator, "
		"third_translator, "
		
		"description "
		"FROM  multi_volume_books_private_data "
		"WHERE "
		"multi_id = %d;", m_book_id);

	sqlite3_prepare (database, sql_query, -1, &first_db_row, 0);
	first_status = sqlite3_step (first_db_row);

	/*	Drugie zapytanie	*/
	g_sprintf (sql_query, "SELECT "
		"same_publish_year, "
		"publish_year, "
		
		"same_translators, "
		"first_translator, "
		"second_translator, "
		"third_translator "
	
		"FROM  multi_volume_books_shared_data "
		"WHERE "
		"id = %d;", m_book_id);
	
	sqlite3_prepare (database, sql_query, -1, &second_db_row, 0);
	second_status = sqlite3_step (second_db_row);
	
	g_printf ("\t<<< First query, SQLite result: %d\n", first_status); 
	g_printf ("\t<<< Second query, SQLite result: %d\n", second_status); 
	
	while (first_status == 100)	{
		MultiVolEditPrivDataStruc	*volume_data;
		
		volume_data = g_new (MultiVolEditPrivDataStruc, 1);
		
		volume_data->id = sqlite3_column_int (first_db_row, 0);
		volume_data->multi_id = sqlite3_column_int (first_db_row, 1);
		g_sprintf (volume_data->subtitle, "%s", sqlite3_column_text (first_db_row, 2));
		g_sprintf (volume_data->orginal_subtitle, "%s", sqlite3_column_text (first_db_row, 3));
		g_sprintf (volume_data->isbn, "%s", sqlite3_column_text (first_db_row, 4));
		/*	Warunkujemy rok wydania	*/
		volume_data->same_publish_year = sqlite3_column_int (second_db_row, 0);
		if (volume_data->same_publish_year == 0)	{
			volume_data->publish_year = sqlite3_column_int (first_db_row, 5);
		}
		else	{
			volume_data->publish_year = sqlite3_column_int (second_db_row, 1);
		}
		
		volume_data->pages = sqlite3_column_int (first_db_row, 6);
		
		/*	Warunkujemy tlumaczy	*/
		volume_data->same_translators = sqlite3_column_int (second_db_row, 2);
		if (volume_data->same_translators == 0)	{
			/*	Pierwszy tlumacza	*/
			if (sqlite3_column_int (first_db_row, 7) > 0)	{
				get_translator_name (item_name, sqlite3_column_int (first_db_row, 7));
				g_sprintf (volume_data->first_translator, "%s", item_name);
			}
			else	{
				g_sprintf (volume_data->first_translator, "%s", "");
			}
			/*	Drugi tlumacza	*/
			if (sqlite3_column_int (first_db_row, 8) > 0)	{
				get_translator_name (item_name, sqlite3_column_int (first_db_row, 8));
				g_sprintf (volume_data->second_translator, "%s", item_name);
			}
			else	{
				g_sprintf (volume_data->second_translator, "%s", "");
			}
			/*	Trzeci tlumacza	*/
			if (sqlite3_column_int (first_db_row, 9) > 0)	{
				get_translator_name (item_name, sqlite3_column_int (first_db_row, 7));
				g_sprintf (volume_data->first_translator, "%s", item_name);
			}
			else	{
				g_sprintf (volume_data->third_translator, "%s", "");
			}
		}
		else	{
			/*	Pierwszy tlumacza	*/
			if (sqlite3_column_int (second_db_row, 3) > 0)	{
				get_translator_name (item_name, sqlite3_column_int (second_db_row, 3));
				g_sprintf (volume_data->first_translator, "%s", item_name);
			}
			else	{
				g_sprintf (volume_data->first_translator, "%s", "");
			}
			/*	Drugi tlumacza	*/
			if (sqlite3_column_int (second_db_row, 4) > 0)	{
				get_translator_name (item_name, sqlite3_column_int (second_db_row, 4));
				g_sprintf (volume_data->second_translator, "%s", item_name);
			}
			else	{
				g_sprintf (volume_data->second_translator, "%s", "");
			}
			/*	Trzeci tlumacza	*/
			if (sqlite3_column_int (second_db_row, 5) > 0)	{
				get_translator_name (item_name, sqlite3_column_int (second_db_row, 5));
				g_sprintf (volume_data->third_translator, "%s", item_name);
			}
			else	{
				g_sprintf (volume_data->third_translator, "%s", "");
			}
		}
		
		g_sprintf (volume_data->description, "%s", sqlite3_column_text (first_db_row, 10));
		/*	Wstawiamy dane do listy	*/
		gtk_list_store_append (GTK_LIST_STORE (store), &iter);
		gtk_list_store_set (GTK_LIST_STORE (store), &iter,
		/*	Wstawiamy id tomu	*/
								0, volume_data->id,
		/*	Wstawiamy podtytul	*/
								1, volume_data->subtitle,
		/*	Wstawiamy adres struktury z danymi	*/
								2, volume_data,
		/*	Domyslnie ksiazka nie jest przeznaczona do usuniecia, wartosc - 0	*/	
								3, 0,
		/*	Domyslnie ksiazka nie zostaje edytowana, wartosc - 0	*/						
								4, 0,
								-1);
		
		/*	Przechodzimy doi nastepnego tomu	*/
		first_status = sqlite3_step (first_db_row);	
	}
	
	sqlite3_finalize (first_db_row);
	sqlite3_finalize (second_db_row);
	
	/*	Aktywujemy okno 'm_book_win'	*/
	set_widget_sensinitive (tmp, "m_book_hbox", TRUE);
}

/*
 *	Funkcja przygotowuje okno dodawania/edycji do uzycia ladujac np. listy autorow, 
 *	gatunkow itd. do odpowiednuch widgetow GtkEntry w oknie.
 *	Funkcja sprawdza, czy w oknie glownym dodawania/edycji ksiazki wielotomowej
 *	zaznaczono opcje jednakowych tlumaczy i daty wdania dla wszystkich tomow.
 *	Jezeli opcje zostaly zaznaczone, to funkcja pobierze odpowiednie dane z okna
 *	nadrzednego "m_book_win" i wstawi je do pol w oknie "m_volume_win"
 */
 
void		m_volume_prepare_win	(GtkWidget	*parent,
									 GtkWidget	*widget)
{
	GladeXML			*parent_xml;
	GladeXML			*widget_xml;
	GtkListStore		*list;
	GtkTreeIter			iter;
	GtkEntryCompletion	*completion;
	GtkWidget			*parent_tmp;
	GtkWidget			*widget_tmp;
	gint				status;
	gint				rows;
	sqlite3_stmt		*db_row;
	
	parent_xml = glade_get_widget_tree (parent);
	widget_xml = glade_get_widget_tree (widget);
	
	parent_tmp = glade_xml_get_widget (parent_xml, "m_book_same_translators_chbox");
	/*	Sprawdzamy czy ustawieni sa wspolni tlumacze	*/
	if ( !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (parent_tmp)))	{
		/*	Ladujemy tlumaczy	*/
		completion = gtk_entry_completion_new ();
		gtk_entry_completion_set_inline_completion (completion, TRUE);
		gtk_entry_completion_set_text_column (completion, 0);
	
		rows = count_returned_rows ("SELECT COUNT(id) FROM translators;");
		g_printf ("\t<<< Loading %d translators...\n", rows);
		if (rows > 0)	{
			sqlite3_prepare (database, "SELECT translator FROM translators;", -1, &db_row, 0);
			status = sqlite3_step (db_row);
			g_printf ("\t<<< Loading table \'translators\', SQLite result: %d\n", status);
		
			list = gtk_list_store_new (1, G_TYPE_STRING);
			if (status == 101)	{
				gtk_list_store_append (list, &iter);
				gtk_list_store_set (list, &iter,
									0, sqlite3_column_text (db_row, 0),
									-1);
			}
			else	{
				while (status == 100)	{
					gtk_list_store_append (list, &iter);
					gtk_list_store_set (list, &iter,
										0, sqlite3_column_text (db_row, 0),
										-1);
					status = sqlite3_step (db_row);
				}
			}
		
			sqlite3_finalize (db_row);
		
			gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (list));	
		
			/*	W koncu ladujemy listy do widgetow...	*/
			/*	Nazwa pola pierwszego tlumacza jest inna w roznych oknach	*/
			widget_tmp = glade_xml_get_widget (widget_xml, "m_volume_first_translator_en");	
			gtk_entry_set_completion (GTK_ENTRY (widget_tmp), completion);
		
			widget_tmp = glade_xml_get_widget (widget_xml, "second_translator_en");	
			completion = gtk_entry_completion_new ();
			gtk_entry_completion_set_inline_completion (completion, TRUE);
			gtk_entry_completion_set_text_column (completion, 0);
			gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (list));	
			gtk_entry_set_completion (GTK_ENTRY (widget_tmp), completion);
		
			widget_tmp = glade_xml_get_widget (widget_xml, "third_translator_en");	
			completion = gtk_entry_completion_new ();
			gtk_entry_completion_set_inline_completion (completion, TRUE);
			gtk_entry_completion_set_text_column (completion, 0);
			gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (list));	
			gtk_entry_set_completion (GTK_ENTRY (widget_tmp), completion);
		}
	}	
	else	{
		parent_tmp = glade_xml_get_widget (parent_xml, "m_same_first_translator_en");
		widget_tmp = glade_xml_get_widget (widget_xml, "m_volume_first_translator_en");
		
		gtk_entry_set_text (GTK_ENTRY (widget_tmp), gtk_entry_get_text (GTK_ENTRY (parent_tmp)));
		set_widget_sensinitive (widget_tmp, "m_volume_first_translator_en", FALSE);
		/*	Dezaktywujemy caly kontener GtkHBox w elemencie GtkExpander w oknie 
		 *	'm_volume_dlg'	*/
		set_widget_sensinitive (widget_tmp, "m_volume_additional_translators_hbox", FALSE);
		/*	Sprawdzamy czy ustawieni sa dodatkowi autorzy, jezeli tak to kopiujemy
		 *	ich pamietajac o wszystkich ustawieniu wszystkich "ptaszkow" itp.	*/
		parent_tmp = glade_xml_get_widget (parent_xml, "same_second_translator_en");
		widget_tmp = glade_xml_get_widget (widget_xml, "second_translator_en");
	
		gtk_entry_set_text (GTK_ENTRY (widget_tmp), gtk_entry_get_text (GTK_ENTRY (parent_tmp)));
			/*	Aktywujemy i otwieramy element GtkExpander w oknie 'm_volume_dlg'	*/
		widget_tmp = glade_xml_get_widget (widget_xml, "m_volume_additional_translators_exp");
		gtk_expander_set_expanded (GTK_EXPANDER (widget_tmp), TRUE);
		set_widget_sensinitive (widget, "m_volume_additional_translators_exp", TRUE);
		
		/*	Sprawdzamy czy ustawieni sa dodatkowi autorzy, jezeli tak to kopiujemy
		 *	ich pamietajac o wszystkich ustawieniu wszystkich "ptaszkow" itp.	*/
		parent_tmp = glade_xml_get_widget (parent_xml, "same_third_translator_en");
		widget_tmp = glade_xml_get_widget (widget_xml, "third_translator_en");
					gtk_entry_set_text (GTK_ENTRY (widget_tmp), gtk_entry_get_text (GTK_ENTRY (parent_tmp)));
		/*	Aktywujemy i otwieramy element GtkExpander w oknie 'm_volume_dlg'	*/
		widget_tmp = glade_xml_get_widget (widget_xml, "m_volume_additional_translators_exp");
		gtk_expander_set_expanded (GTK_EXPANDER (widget_tmp), TRUE);
		set_widget_sensinitive (widget, "m_volume_additional_translators_exp", TRUE);
	}
	
	parent_tmp = glade_xml_get_widget (parent_xml, "m_book_same_publish_year_check");
	/*	Sprawdzamy czy ustawiony jest wpolny rok wydania	*/
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (parent_tmp)))	{
		parent_tmp = glade_xml_get_widget (parent_xml, "same_publish_year_spin");
		widget_tmp = glade_xml_get_widget (widget_xml, "publish_year_spin");
		/*	Kopiujemy rok wydania z okna do okna	*/
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget_tmp),
		gtk_spin_button_get_value (GTK_SPIN_BUTTON (parent_tmp)));
		/*	Haszujemy widget, zeby nie mozna bylo goedytowac	*/
		set_widget_sensinitive (widget_tmp, "publish_year_spin", FALSE);
	}
	
	set_widget_sensinitive (widget, "m_volume_hbox", TRUE);
}

/*
 *	Funkcja sprawdza poprawnosc wprowadzonych danych w oknie 'm_volume_dlg'
 */

gboolean	m_volume_check_data		(GtkWidget	*widget)	{
	GladeXML		*xml;
	GtkWidget		*tmp;
	GtkTextBuffer 	*text_buffer;
	GtkTextIter		text_start;	
	GtkTextIter		text_end;
	gchar			string 			[VERY_SHORT_STRING];
	gint			i;
	
	i = 0;
	
	xml = glade_get_widget_tree (widget);
	/*	Pole 'Title' nie moze byc puste	*/
	tmp = glade_xml_get_widget (xml, "subtitle_en");
	if (strlen ( gtk_entry_get_text (GTK_ENTRY (tmp))) == 0)	{
		show_message_dlg (_("Invalid field value"), _("Field \'Subtitle\' can't be empty!"), 1);
		return FALSE;
	}
	
	/*	Sprawdzamy czy pole 'ISBN' jest poprawnie wypelnione	*/
	tmp = glade_xml_get_widget (xml, "isbn_en");
	
	if (strlen ( gtk_entry_get_text (GTK_ENTRY (tmp))) == 0)
	{
			/*	Nic nie robimy	*/
	}	
	else if (strlen (gtk_entry_get_text (GTK_ENTRY (tmp))) < 10)	{
		show_message_dlg (_("Invalid field value"), _("ISBN number is too short"), 1);
		return FALSE;
	}
	else if	(strlen (gtk_entry_get_text (GTK_ENTRY (tmp))) == 10)	{
	/*	Wszystkie znaki w plu ISBN musza byc cyframi	*/
		g_sprintf (string, "%s", gtk_entry_get_text (GTK_ENTRY (tmp)));
		while (i < 10)	{
			if (g_ascii_isdigit (string [i]) == FALSE)	{
				show_message_dlg (_("Invalid field value"), _("Field \'ISBN\' can't "
								"contain other value than numeric signs!"), 1);
				return FALSE;
			}
			++i;
		}
	}
	
	/*	Sprawdzamy dlugosc testu w polu 'Description'	*/
	tmp = glade_xml_get_widget (xml, "description_text");
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tmp));
	gtk_text_buffer_get_bounds (text_buffer, &text_start, &text_end);	
	if (strlen ( gtk_text_buffer_get_text (text_buffer, &text_start, &text_end, TRUE)) > 2000)	{
		show_message_dlg (_("Invalid field value"), _("Description can't be longer than 2000 signs!"), 1);
		return FALSE;
	}
	
	return TRUE;
}

/*
 *	Funkcja dodaje nowy tom do listy w'books_list' w oknie 'm_book_win'
 */

void	m_volume_add_book	(GtkWidget			*parent,
							 GtkWidget			*widget)
{
	MultiVolEditPrivDataStruc	*volume_data;
	GladeXML					*parent_xml;
	GladeXML					*widget_xml;
	GtkTextBuffer				*text_buffer;
	GtkTextIter					text_start;	
	GtkTextIter					text_end;
	GtkWidget					*tmp;
	GtkTreeModel				*store;
	GtkTreeIter 				iter;
	
	parent_xml = glade_get_widget_tree (parent);
	widget_xml = glade_get_widget_tree (widget);
	
	/*	Zajmujemy miejsce w pamieci dla nowej struktury	*/
	volume_data = g_new (MultiVolEditPrivDataStruc, 1);
	
	/*	Kopiujemy dane z okna do struktury	*/
	tmp = glade_xml_get_widget (widget_xml, "subtitle_en");
	g_sprintf (volume_data->subtitle, "%s", gtk_entry_get_text (GTK_ENTRY (tmp)));
	
	tmp = glade_xml_get_widget (widget_xml, "orginal_subtitle_en");
	g_sprintf (volume_data->orginal_subtitle, "%s", gtk_entry_get_text (GTK_ENTRY (tmp)));
	
	tmp = glade_xml_get_widget (widget_xml, "isbn_en");
	g_sprintf (volume_data->isbn, "%s", gtk_entry_get_text (GTK_ENTRY (tmp)));
	
	tmp = glade_xml_get_widget (widget_xml, "publish_year_spin");
	volume_data->publish_year = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tmp));	
	
	tmp = glade_xml_get_widget (widget_xml, "pages_number_spin");
	volume_data->pages = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tmp));
	
	tmp = glade_xml_get_widget (widget_xml, "m_volume_first_translator_en");
	g_sprintf (volume_data->first_translator, "%s", gtk_entry_get_text (GTK_ENTRY (tmp)));
	
	tmp = glade_xml_get_widget (widget_xml, "second_translator_en");
	g_sprintf (volume_data->second_translator, "%s", gtk_entry_get_text (GTK_ENTRY (tmp)));
		
	tmp = glade_xml_get_widget (widget_xml, "third_translator_en");
	g_sprintf (volume_data->third_translator, "%s", gtk_entry_get_text (GTK_ENTRY (tmp)));
	
	tmp = glade_xml_get_widget (widget_xml, "description_text");
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tmp));
	gtk_text_buffer_get_bounds (text_buffer, &text_start, &text_end);
	g_sprintf (volume_data->description, "%s", gtk_text_buffer_get_text (text_buffer, &text_start, &text_end, TRUE));
	
	tmp = glade_xml_get_widget (parent_xml, "books_list");
	store = gtk_tree_view_get_model (GTK_TREE_VIEW (tmp));
	
	gtk_list_store_append (GTK_LIST_STORE (store), &iter);
	gtk_list_store_set (GTK_LIST_STORE (store), &iter,
		/*	Nie jestesmy w sstanie odac ID bo ksiazki nie ma w bazie danych	*/
		/*						0, volume_data->id,							*/
		/*	Wstawiamy podtytul	*/
								1, volume_data->subtitle,
		/*	Wstawiamy adres struktury z danymi	*/
								2, volume_data,
		/*	Domyslnie ksiazka nie jest przeznaczona do usuniecia, wartosc - 0	*/	
								3, 0,
		/*	Domyslnie ksiazka nie zostaje edytowana, wartosc - 0	*/						
								4, 0,
								-1);					
}

/*
 *	Funkcja laduje dane o tomie do okna edycji 'm_volume_win'
 */

void	m_volume_load_win		(GtkWidget	*widget,
								 GtkWidget	*parent)
{
	MultiVolEditPrivDataStruc	*volume_data;
	GladeXML					*parent_xml;
	GladeXML					*widget_xml;
	GtkTextBuffer				*text_buffer;
	GtkWidget					*parent_tmp;
	GtkWidget					*tmp;
	GtkTreeSelection			*selection;
	GtkTreeModel				*model;
  	GtkTreeIter					iter;
	gint 						adress;
	gchar						string	[SHORT_STRING];	
	
	parent_xml = glade_get_widget_tree (parent);
	widget_xml = glade_get_widget_tree (widget);
	
	/*===	Wyciagamy adres struktury	===*/
	tmp = glade_xml_get_widget (parent_xml, "books_list"); 
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tmp));
	gtk_tree_selection_get_selected (selection, &model, &iter);
	
	/*	Pibieramy zaznaczony wiersz i wyciagamy z niego zawartosc drugiej 
	 *	kolumny, przechowujacej adres struktury z danymi i tomie	*/
	gtk_tree_model_get (model, &iter,
		2, &adress,
		-1);
	
	volume_data = (MultiVolEditPrivDataStruc *) adress;
	/*	Tytul	*/
	tmp = glade_xml_get_widget (widget_xml, "subtitle_en");
	gtk_entry_set_text (GTK_ENTRY (tmp), volume_data->subtitle);
	/*	Orginalny tytul	*/
	tmp = glade_xml_get_widget (widget_xml, "orginal_subtitle_en");
	gtk_entry_set_text (GTK_ENTRY (tmp), volume_data->orginal_subtitle);
	/*	Numer ISBN	*/
	tmp = glade_xml_get_widget (widget_xml, "isbn_en");
	gtk_entry_set_text (GTK_ENTRY (tmp), volume_data->isbn);
	/*	Data wydania	*/
	if (volume_data->same_publish_year == 0)	{
		/*	Ustawiony jest wspolny rok wydania, wiec dezaktywujemy widget 
		 *	'pages_number_spin' i pobieramy dane z okna 'm_book_win'	*/
		tmp = glade_xml_get_widget (widget_xml, "publish_year_spin");
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (tmp), volume_data->publish_year);		
	}
	/*	Ilosc stron	*/
	tmp = glade_xml_get_widget (widget_xml, "pages_number_spin");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (tmp), volume_data->pages);
	
	/*==============*/
	/*	Tlumacze	*/
	/*==============*/
	
	/*	Jezeli ustawieni sa wspolni tlumacze (wartosc 1) to nie ladujemy tlumaczy
	 *	ze struktury	*/
	if (volume_data->same_translators == 0)	{
		/*	Tlumacze nie sa wspolni, wiec ladujemy ich z listy, pierwszy tlumacz	*/
		tmp = glade_xml_get_widget (widget_xml, "m_volume_first_translator_en");
		gtk_entry_set_text (GTK_ENTRY (tmp), volume_data->first_translator);

		if (strlen (volume_data->first_translator) > 0)	{
			/*	Drugi tłumacz	*/
			if (strlen (volume_data->second_translator) > 0)	{
				tmp = glade_xml_get_widget (widget_xml, "second_translator_en");		
				gtk_entry_set_text (GTK_ENTRY (tmp), volume_data->second_translator);
				/*	Aktywujemy i otwieramy element GtkExpander w oknie 'm_volume_dlg'	*/
				tmp = glade_xml_get_widget (widget_xml, "m_volume_additional_translators_exp");
				gtk_expander_set_expanded (GTK_EXPANDER (tmp), TRUE);
				set_widget_sensinitive (tmp, "m_volume_additional_translators_exp", TRUE);
			}
			/*	Trzeci tlumacz	*/
			if (strlen (volume_data->third_translator) > 0)	{
				tmp = glade_xml_get_widget (widget_xml, "third_translator_en");			
				gtk_entry_set_text (GTK_ENTRY (tmp), volume_data->third_translator);
				/*	Otwieramy element GtkExpander w oknie 'm_volume_dlg'	*/
				tmp = glade_xml_get_widget (widget_xml, "m_volume_additional_translators_exp");
				gtk_expander_set_expanded (GTK_EXPANDER (tmp), TRUE);
				set_widget_sensinitive (tmp, "m_volume_additional_translators_exp", TRUE);
			}
		}	
	}
	else	{
		parent_tmp = glade_xml_get_widget (parent_xml, "m_same_first_translator_en");
		tmp = glade_xml_get_widget (widget_xml, "m_volume_first_translator_en");
		
		gtk_entry_set_text (GTK_ENTRY (tmp), gtk_entry_get_text (GTK_ENTRY (parent_tmp)));
		set_widget_sensinitive (tmp, "m_volume_first_translator_en", FALSE);
		/*	Dezaktywujemy caly kontener GtkHBox w elemencie GtkExpander w oknie 
		 *	'm_volume_dlg'	*/
		set_widget_sensinitive (tmp, "m_volume_additional_translators_hbox", FALSE);
		/*	Sprawdzamy czy ustawieni sa dodatkowi autorzy, jezeli tak to kopiujemy
		 *	ich pamietajac o wszystkich ustawieniu wszystkich "ptaszkow" itp.	*/
		parent_tmp = glade_xml_get_widget (parent_xml, "same_second_translator_en");
		tmp = glade_xml_get_widget (widget_xml, "second_translator_en");
	
		gtk_entry_set_text (GTK_ENTRY (tmp), gtk_entry_get_text (GTK_ENTRY (parent_tmp)));
			/*	Aktywujemy i otwieramy element GtkExpander w oknie 'm_volume_dlg'	*/
		tmp = glade_xml_get_widget (widget_xml, "m_volume_additional_translators_exp");
		gtk_expander_set_expanded (GTK_EXPANDER (tmp), TRUE);
		set_widget_sensinitive (widget, "m_volume_additional_translators_exp", TRUE);
		
		/*	Sprawdzamy czy ustawieni sa dodatkowi autorzy, jezeli tak to kopiujemy
		 *	ich pamietajac o wszystkich ustawieniu wszystkich "ptaszkow" itp.	*/
		parent_tmp = glade_xml_get_widget (parent_xml, "same_third_translator_en");
		tmp = glade_xml_get_widget (widget_xml, "third_translator_en");
					gtk_entry_set_text (GTK_ENTRY (tmp), gtk_entry_get_text (GTK_ENTRY (parent_tmp)));
		/*	Aktywujemy i otwieramy element GtkExpander w oknie 'm_volume_dlg'	*/
		tmp = glade_xml_get_widget (widget_xml, "m_volume_additional_translators_exp");
		gtk_expander_set_expanded (GTK_EXPANDER (tmp), TRUE);
		set_widget_sensinitive (widget, "m_volume_additional_translators_exp", TRUE);
	}
	
	/*	Opis	*/
	tmp = glade_xml_get_widget (widget_xml, "description_text");
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tmp));
	gtk_text_buffer_set_text (text_buffer, volume_data->description, strlen (volume_data->description));
}

/*
 *	Funkcja wywolywana jest po zakonczeniu edycji danego tomu. Kopiuje ona dane z
 *	okna 'm_volume_win' do sturktury zawierajacej dane o edytowanym tomie. Adres
 *	struktury jest pobierany z ukrytego pola w oknie 'm_volume_win'.
 */

void		m_volume_update_book	(GtkWidget		*parent,
									 GtkWidget		*widget)
{
	MultiVolEditPrivDataStruc	*volume_data;
	GladeXML					*parent_xml;
	GladeXML					*widget_xml;
	GtkTreeSelection			*selection;
	GtkTreeModel				*model;
  	GtkTreeIter					iter;
	GtkTextBuffer				*text_buffer;
	GtkTextIter					text_start;	
	GtkTextIter					text_end;
	GtkWidget					*tmp;
	gint						adress;
	
	parent_xml = glade_get_widget_tree (parent);
	widget_xml = glade_get_widget_tree (widget);
	
	/*	Wyciagamy adres struktury	*/
	tmp = glade_xml_get_widget (parent_xml, "books_list"); 
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tmp));
	gtk_tree_selection_get_selected (selection, &model, &iter);
	
	gtk_tree_model_get (model, &iter,
		2, &adress,
		-1);
	
	volume_data = (MultiVolEditPrivDataStruc *) adress;
	
	tmp = glade_xml_get_widget (widget_xml, "subtitle_en");
	g_sprintf (volume_data->subtitle, "%s", gtk_entry_get_text (GTK_ENTRY (tmp)));
	
	tmp = glade_xml_get_widget (widget_xml, "orginal_subtitle_en");
	g_sprintf (volume_data->orginal_subtitle, "%s", gtk_entry_get_text (GTK_ENTRY (tmp)));
	
	tmp = glade_xml_get_widget (widget_xml, "isbn_en");
	g_sprintf (volume_data->isbn, "%s", gtk_entry_get_text (GTK_ENTRY (tmp)));
	printf ("\n  \"%s  \"\n", volume_data->isbn);
	/*	Aktualizujemy rok wydania tylko wtedy, gdy nie jest ustawiony wspolny roku
	 *	wydania dla wszystkich tomow	*/
	if (volume_data->same_publish_year == 0)	{
		tmp = glade_xml_get_widget (widget_xml, "publish_year_spin");
		volume_data->publish_year = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tmp));	
	}
	
	tmp = glade_xml_get_widget (widget_xml, "pages_number_spin");
	volume_data->pages = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tmp));
	
	/*	Aktualizujemy tlumaczy tylko wtedy, gdy nie sa ustawieni wspolni tlumacze
	 *	dla wszystkich tomow	*/
	if (volume_data->same_translators == 0)	{
		tmp = glade_xml_get_widget (widget_xml, "m_volume_first_translator_en");
		g_sprintf (volume_data->first_translator, "%s", gtk_entry_get_text (GTK_ENTRY (tmp)));
		
		if (strlen (gtk_entry_get_text (GTK_ENTRY (tmp))) > 0)	{
			tmp = glade_xml_get_widget (widget_xml, "second_translator_en");
			g_sprintf (volume_data->second_translator, "%s", gtk_entry_get_text (GTK_ENTRY (tmp)));
		
			tmp = glade_xml_get_widget (widget_xml, "third_translator_en");
			g_sprintf (volume_data->third_translator, "%s", gtk_entry_get_text (GTK_ENTRY (tmp)));
		}
	}
	
	tmp = glade_xml_get_widget (widget_xml, "description_text");
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tmp));
	gtk_text_buffer_get_bounds (text_buffer, &text_start, &text_end);
	g_sprintf (volume_data->description, "%s", gtk_text_buffer_get_text (text_buffer, &text_start, &text_end, TRUE));

	/*	Tom byl edytowany, wiec zmieniamy wartosc ostatniej kolumny listy
	 *	'books_list'	*/
	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
								4, 1,
								-1);
}

/*
 *	Funkcja sprawdza czy tlumacze dodani do listy 'books_list' w oknie
 *	'm_books_list' poprzez okno 'm_voluime_win' znajduja sie w bazie danych.
 *	Jezeli nie ma ich to, to zostaje wyswietlone pytanie, czy user chce ich
 *	teraz dodac. Funkcja nie urochomi sie jezeli zostali ustawieni wspolni
 *	tlumacze.
 *
 *	Funkcja uruchamiana jest tylko w systuacji, gdy nie sa ustawieni wspolni 
 *	tlumacze!
 */

void					m_volume_check_linked_data		(GtkWidget		*widget)	{
	GladeXML					*xml;
	GtkTreeModel				*model;
  	GtkTreeIter					iter;
	GtkWidget					*tmp;
	gchar						sql_query			[SHORT_STRING];
	gchar						string				[SHORT_STRING];
	gint						adress;
	gint						status;
	gint						rows;
	gboolean					result;
	sqlite3_stmt				*db_row;
	
	xml = glade_get_widget_tree (widget);
	
	/*	Sprawdzamy, czy ustawiono wspolych tlumaczy, jezeli tak to wychodzimy 
	 *	z funkcji	*/
	tmp = glade_xml_get_widget (xml, "m_book_same_translators_chbox");
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
		return;
	}
	
	tmp = glade_xml_get_widget (xml, "books_list");
	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tmp));
	
	result = gtk_tree_model_get_iter_first (model, &iter);
	/*	result musi zwrocic TRUE bo lista byla juz sprawdzana	*/
	while	(result == TRUE)	{
		MultiVolEditPrivDataStruc	*volume_data;
		
		gtk_tree_model_get (model, &iter,
				2, &adress, 
				-1);
		
		volume_data = (MultiVolEditPrivDataStruc *) adress;
		
		/*	Pierwszy tlumacz	*/
		if (strlen (volume_data->first_translator) > 0)	{
			g_sprintf (sql_query, "SELECT COUNT(id) FROM translators WHERE translator=\'%s\';",
					volume_data->first_translator);
			rows = count_returned_rows (sql_query);
			if (rows == 0)	{
				g_sprintf (string, _("There is no translator \'%s\' in database, do you want "
						"to add it now?"), volume_data->first_translator);
				result = show_question_dlg (_("New translator"), string, 0);
				if (result == TRUE)	{
					g_sprintf (sql_query, "INSERT INTO translators (id, translator) VALUES (NULL, \'%s\');",
						volume_data->first_translator);
					sqlite3_prepare (database, sql_query, -1, &db_row, 0);
					status = sqlite3_step (db_row);
					g_printf ("\t\t>>> Adding new translator, SQLite result: %d\n", status);
					sqlite3_finalize (db_row);
				}
			}
		}
		
		/*	Drugi tlumacz	*/
		if (strlen (volume_data->second_translator) > 0)	{
			g_sprintf (sql_query, "SELECT COUNT(id) FROM translators WHERE translator=\'%s\';",
					volume_data->second_translator);
			rows = count_returned_rows (sql_query);
			if (rows == 0)	{
				g_sprintf (string, _("There is no translator \'%s\' in database, do you want "
						"to add it now?"), volume_data->second_translator);
				result = show_question_dlg (_("New translator"), string, 0);
				if (result == TRUE)	{
					g_sprintf (sql_query, "INSERT INTO translators (id, translator) VALUES (NULL, \'%s\');",
						volume_data->second_translator);
					sqlite3_prepare (database, sql_query, -1, &db_row, 0);
					status = sqlite3_step (db_row);
					g_printf ("\t\t>>> Adding new translator, SQLite result: %d\n", status);
					sqlite3_finalize (db_row);
				}
			}
		}
	
		/*	Trzeci tlumacz	*/
		if (strlen (volume_data->third_translator) > 0)	{
			g_sprintf (sql_query, "SELECT COUNT(id) FROM translators WHERE translator=\'%s\';",
					volume_data->third_translator);
			rows = count_returned_rows (sql_query);
			if (rows == 0)	{
				g_sprintf (string, _("There is no translator \'%s\' in database, do you want "
						"to add it now?"), volume_data->third_translator);
				result = show_question_dlg (_("New translator"), string, 0);
				if (result == TRUE)	{
					g_sprintf (sql_query, "INSERT INTO translators (id, translator) VALUES (NULL, \'%s\');",
						volume_data->third_translator);
					sqlite3_prepare (database, sql_query, -1, &db_row, 0);
					status = sqlite3_step (db_row);
					g_printf ("\t\t>>> Adding new translator, SQLite result: %d\n", status);
					sqlite3_finalize (db_row);
				}
			}
		}
		
		/*	Pobieramy nastepny wiersz	*/
		result = gtk_tree_model_iter_next (model, &iter);
	}	
}

/*
 *	Funkcja zapisuje do bazy danych wszystkie tomy znajdujace sie w liscie
 *	'books_list' w oknie 'm_book_win'. Przy okazji funkcja sprawdza, czy wybrani
 *	w oknie 'm_volume_win' tlumacze sa juz w bazie danych, jeze li nie to ich 
 *	dodaje.
 *	Funkcja dziala w dwoch trybach, bo dodawanie tomow w czasie edycji i 
 *	dodawania ksiazki wielotomowej rozni sie od siebie.
 *	
 *	Wartosci argumentu 'mode':
 *	- 0 - dodawanie ksiazki wielotomowej
 *	- 1 - edycja ksiazki wielotomowej
 *
 *	=================================================================
 *	>>> Funkcja musi byc wywolana po funkcji 'm_insert_book ()'	! <<<
 *	>>> gdy dodajemy nowa ksiazke								  <<<
 *	=================================================================
 */

void	m_volume_insert_volumes	(GtkWidget	*parent, gint	mode)	{
	GladeXML			*xml;
	GtkWidget			*tmp;
	GtkTreeSelection	*selection;
	GtkTreeModel		*model;
  	GtkTreeIter			iter;
	gchar				sql_query 		[VERY_LONG_STRING];
	gint				adress;
	gint				status;
	gint				rows;
	gint				multi_id;
	gint				publish_year;
	gint				first_translator;
	gint				second_translator;
	gint				third_translator;
	gint				remove_status;
	gint				volume_id;
	gboolean			result;
	sqlite3_stmt		*db_row;
	
	xml = glade_get_widget_tree (parent);
	
	switch (mode)	{
		case 0:
			/*	Pobieramy ID nowej ksiazki wielotomowej	*/
			sqlite3_prepare (database, "SELECT MAX(id) FROM multi_volume_books_shared_data;", -1, &db_row, 0);
			status = sqlite3_step (db_row);
			g_printf ("\t <<< Selecting new book\'s ID, SQLite result: %d\n", status);
			multi_id = sqlite3_column_int (db_row, 0);
			sqlite3_finalize (db_row);
			break;
		case 1:
			/*	Pobieramy ID nowej ksiazki wielotomowej	*/
			tmp = glade_xml_get_widget (xml, "book_id_lb");
			sscanf (gtk_label_get_text (GTK_LABEL (tmp)), "%d", &multi_id);
			break;
	}		
	
	/*===	Wyciagamy adres struktury	===*/
	tmp = glade_xml_get_widget (xml, "books_list"); 
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tmp));
	gtk_tree_selection_get_selected (selection, &model, &iter);
	
	result = gtk_tree_model_get_iter_first (model, &iter);
	while (result == TRUE)	{
		MultiVolEditPrivDataStruc	*volume_data;
		/*	
		 *	Jezeli dodajemy tomy w trybie edycji, musza zostac spelnione
		 *	nastepujace warunki zeby ksiazka byla dodana:
		 *	- wartosc pierwszej kolumny (numer ID) musi wynosic 0
		 *	- wartosc czwartej kolumny musi miec wartosc 0 (ksiazka nie moze 
		 *	  byc dodana gdy jest przeznaczona do usuniecia)
		 *	Jezeli natomiast funkcja jest wywolana w trybie dodawania, dodawane sa
		 *	po prostu wszystkie ksiazki z uwzglednieniem wartosci 0 dla czwartej
		 *	kolumny w liscie.
		 */
		
		/*	Pobieramy potrzebvne dane z listy, zaleznie od trybu	*/
		switch (mode)	{
			case 0:
				gtk_tree_model_get (model, &iter,
					2, &adress,
					3, &remove_status,
					-1);
				break;
			case 1:
				gtk_tree_model_get (model, &iter,
					0, &volume_id,
					2, &adress,
					3, &remove_status,
					-1);
				break;
		}

		/*	Pora sprawdzic powyzsze zalozenia podane w komentarzu, na poczatku
		 *	tej petli.	*/
		if (mode == 0 && remove_status == 0)	{
			/*	Nic nie robimy	*/
		}
		else if (mode == 1 && 
				 remove_status == 0	&&
				 volume_id == 0)
		{
			/*	Nic nie robimy	*/
		}
		else {
			/*	Jezeli dojdziemy tutaj, oznacza, to ze powyzsze kryteria nie 
			 *	zostaly spelnione dla tego tomu, wiec przechodzimy do 
			 *	nastepnego tomu	*/
			
			/*	Pobieramy nastepny wiersz	*/
			result = gtk_tree_model_iter_next (model, &iter);
			
			continue;
		}
		
		volume_data = (MultiVolEditPrivDataStruc *) adress;
		/*	Ustalamy rok wydania	*/
		if (volume_data->same_publish_year == 1)	{
			tmp = glade_xml_get_widget (xml, "same_publish_year_spin");
			publish_year = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tmp));
		}
		else	{
			publish_year = volume_data->publish_year;
		}
		
		/*	Ustalamy numery id tlumaczy	*/
		if (volume_data->same_translators == 1)	{
			tmp = glade_xml_get_widget (xml, "m_same_first_translator_en");
			if (strcmp (gtk_entry_get_text (GTK_ENTRY (tmp)), "") == 0)	{
				first_translator = 0;
			}
			else	{
				first_translator = get_translator_id (gtk_entry_get_text (GTK_ENTRY (tmp)));
			}
			
			/*	Drugi tlumacz	*/
			tmp = glade_xml_get_widget (xml, "same_second_translator_en");
			if (strcmp (gtk_entry_get_text (GTK_ENTRY (tmp)), "") == 0)	{
				second_translator = 0;
			}
			else	{
				second_translator = get_translator_id (gtk_entry_get_text (GTK_ENTRY (tmp)));
			}
			
			/*	Trzeci tlumacz	*/
			tmp = glade_xml_get_widget (xml, "same_third_translator_en");
			if (strcmp (gtk_entry_get_text (GTK_ENTRY (tmp)), "") == 0)	{
				third_translator = 0;
			}
			else	{
				third_translator = get_translator_id (gtk_entry_get_text (GTK_ENTRY (tmp)));
			}
		}
		else	{
			if (strlen (volume_data->first_translator) == 0)	{
				first_translator = 0;
			}
			else	{
				first_translator = get_translator_id (volume_data->first_translator);
			}
			/*	Drugi tlumacz	*/
			if (strlen (volume_data->second_translator) == 0)	{
				second_translator = 0;
			}
			else	{
				second_translator = get_translator_id (volume_data->second_translator);
			}
			/*	Trzeci tlumacz	*/
			if (strlen (volume_data->third_translator) == 0)	{
				third_translator = 0;
			}
			else	{
				third_translator = get_translator_id (volume_data->third_translator);
			}
		}
	
		g_sprintf (sql_query, "INSERT INTO multi_volume_books_private_data "
			"(id, "
			"multi_id, "
			"title, "
			"orginal_title, "
			"isbn, "
			"publish_year, "
			"pages, "
			"first_translator, "
			"second_translator, "
			"third_translator, "
			"description) "
	
			"VALUES "
	
			"(NULL, "
			"%d, "
			"\'%s\', "
			"\'%s\', "
			"\'%s\', "
			"%d, "
			"%d, "
			"%d, "
			"%d, "	
			"%d, "
			"\'%s\');", 
			multi_id,
			volume_data->subtitle,
			volume_data->orginal_subtitle,
			volume_data->isbn,
			publish_year,
			volume_data->pages,
			first_translator,
			second_translator,
			third_translator,
			volume_data->description);	
		
		/*	Dodajamy poszczegolne tomy	*/
		sqlite3_prepare (database, sql_query, -1, &db_row, 0);
		status = sqlite3_step (db_row);
		g_printf ("\t <<< Adding new volume, SQLite result: %d\n", status);
		sqlite3_finalize (db_row);	
		
		/*	Pobieramy nastepny wiersz	*/
		result = gtk_tree_model_iter_next (model, &iter);
	}
}

/*
 *	W liscie 'books_list' w oknie 'm_book_win' druga kolumna przechowuje adresy
 *	do wskaznikow struktur przechowujacych dane o poszczegolnych tomach (podtytul,
 *	liczba stron itp.). Przy konczeniu dodawania/edycji ksiazki wielotomowej
 *	wypadaloby zwolnic zajeta pamiec i tym wlasnie zajmuje sie ta funkcja.
 */

void	m_volume_free_memory		(GtkWidget		*widget)	{
	GladeXML		*xml;
	GtkTreeModel	*model;
  	GtkTreeIter		iter;
	GtkWidget		*tmp;
	gint			adress;
	gint			i;
	gboolean		result;

	xml = glade_get_widget_tree (widget);
	i = 0;
	
	tmp = glade_xml_get_widget (xml, "books_list");
	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tmp));

	result = gtk_tree_model_get_iter_first (model, &iter);
	while (result == TRUE)	{		
		gtk_tree_model_get (model, &iter,
			2, &adress,
			-1);
		
		g_free ((MultiVolEditPrivDataStruc *)adress);
		i++;
		/*	Pobieramy nastepny wiersz	*/
		result = gtk_tree_model_iter_next (model, &iter);
	}
	printf ("\t\t<<< Freeing memory, removed %d structures.\n", i);
}

/*
 *	Poszczegolne tomy mozna usuwac wprost z okna 'm_book_win' i wlasnie tym
 *	zajmuje sie ta funkcja. Tomy przeznaczone do usuniecia sa zanzaczone na
 *	czerwono w lisie w oknie 'm_book_win'.
 */

void	m_volume_remove_volumes	(GtkWidget	*widget)	{
	GladeXML		*xml;
	GtkTreeModel	*model;
  	GtkTreeIter		iter;
	GtkWidget		*tmp;
	gchar			string		[MEDIUM_STRING];
	gint			status;
	gint			volume_id;
	gint			remove_status;
	gint			volumes_number;
	gint			end_volumes_number;
	gint			multi_id;
	gint			i;
	gboolean		result;
	sqlite3_stmt	*db_row;

	xml = glade_get_widget_tree (widget);
	i = 0;
	
	tmp = glade_xml_get_widget (xml, "books_list");
	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tmp));

	result = gtk_tree_model_get_iter_first (model, &iter);
	while (result == TRUE)	{
		/*	Trzecia kolumna listy (liczac od 0) przechowuje wartosc 1 lub 0:
		 *	- 0 - gdy ksiazka nie ma byc usunieta
		 *	- 1 - gdy ksiazka ma byc usunieta
		 */		
		gtk_tree_model_get (model, &iter,
			3, &remove_status,
			-1);		
		if (remove_status == 1)	{
		/*	Liczymu ksiazki, ktore maja byc skasowane	*/
			++i;
		}		
		/*	Pobieramy nastepny wiersz	*/
		result = gtk_tree_model_iter_next (model, &iter);
	}
	
	/*	Upewaniamy sie czy na pewno ksiazki maja byc usuniete	*/
	if (i > 0)	{
		g_sprintf (string, _("Do you really want to remove %d volume(s) from database? "
			"Remember, that you want be able to restore them."), i);
	
		result = show_question_dlg (_("Remove volumes"), string, 2);
		if (result == TRUE)	{
			/*	Pobieramy numer ID ksiazki wielotomowej	*/
			tmp = glade_xml_get_widget (xml, "book_id_lb");
			sscanf (gtk_label_get_text (GTK_LABEL (tmp)), "%d", &multi_id);
			/*	Pamietamy, ze ksiazka wielotomowa musi zawierac minumum dwa tomy.
			 *	Pora na odrobine matematyki	;-)	*/
			g_sprintf (string, "SELECT COUNT(id) FROM multi_volume_books_private_data WHERE multi_id=%d;", multi_id);
			volumes_number = count_returned_rows (string);
			
			end_volumes_number = volumes_number - i;
			/*	Ksiazka wielotomowa musi zawierac minium dwa tomy	*/
			if (end_volumes_number < 2)	{
				g_sprintf (string, _("Can't remove %d volume(s), "
					"becasue multi volume book must contain minumum two volumes!."),
					i);
				show_message_dlg (_("Update error"), string, 2);
			}
			else	{
				result = gtk_tree_model_get_iter_first (model, &iter);
				while (result == TRUE)	{
					gtk_tree_model_get (model, &iter,
						0, &volume_id, 
						3, &remove_status,
						-1);		
						
					if (remove_status == 1)	{
						g_sprintf (string, "DELETE FROM multi_volume_books_private_data WHERE id=%d;",
							volume_id);
						sqlite3_prepare (database, string, -1, &db_row, 0);
						status = sqlite3_step (db_row);
						g_printf ("\t<<< Removing volume with ID: %d, SQLite result: %d\n", volume_id, status);
						sqlite3_finalize (db_row);
					}
					
					/*	Pobieramy nastepny wiersz	*/
					result = gtk_tree_model_iter_next (model, &iter);
				}
			}
		}
	}
}

/*
 *	Funkcja zapisuje w bazie danych zmodyfikowane informacje o tomach, ktore 
 *	byly edytowane w oknie 'm_book_win'. 
 */

void	m_volume_update_volumes_data	(GtkWidget		*widget)	{
	GladeXML		*xml;
	GtkTreeModel	*model;
  	GtkTreeIter		iter;
	GtkWidget		*tmp;
	gchar			sql_query		[VERY_LONG_STRING];
	gint			status;
	gint			volume_id;
	gint			adress;
	gint			update_status;
	gint			rows;
	gint			publish_year;
	gint			first_translator;
	gint			second_translator;
	gint			third_translator;
	gboolean		result;
	sqlite3_stmt	*db_row;

	xml = glade_get_widget_tree (widget);
	
	tmp = glade_xml_get_widget (xml, "books_list");
	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tmp));

	result = gtk_tree_model_get_iter_first (model, &iter);
	while (result == TRUE)	{
		/*	czwarta kolumna listy (liczac od 0) przechowuje wartosc 1 lub 0:
		 *	- 0 - gdy ksiazka nie byla edytowana
		 *	- 1 - gdy ksiazka byla edytowana
		 */		
		gtk_tree_model_get (model, &iter,
			0, &volume_id,
			2, &adress,
			4, &update_status,
			-1);		
		if (update_status == 1)	{
			MultiVolEditPrivDataStruc	*volume_data;
			
			volume_data = (MultiVolEditPrivDataStruc *) adress;
			
			/*	Ustalamy rok wydania	*/
			if (volume_data->same_publish_year == 1)	{
				tmp = glade_xml_get_widget (xml, "same_publish_year_spin");
				publish_year = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tmp));
			}
			else	{
				publish_year = volume_data->publish_year;
			}
			
			/*	Ustalamy numery id tlumaczy	*/
			if (volume_data->same_translators == 1)	{
				tmp = glade_xml_get_widget (xml, "m_same_first_translator_en");
				if (strcmp (gtk_entry_get_text (GTK_ENTRY (tmp)), "") == 0)	{
					first_translator = 0;
				}
				else	{
					first_translator = get_translator_id (gtk_entry_get_text (GTK_ENTRY (tmp)));
				}
				
				/*	Drugi tlumacz	*/
				tmp = glade_xml_get_widget (xml, "same_second_translator_en");
				if (strcmp (gtk_entry_get_text (GTK_ENTRY (tmp)), "") == 0)	{
					second_translator = 0;
				}
				else	{
					second_translator = get_translator_id (gtk_entry_get_text (GTK_ENTRY (tmp)));
				}
				
				/*	Trzeci tlumacz	*/
				tmp = glade_xml_get_widget (xml, "same_third_translator_en");
				if (strcmp (gtk_entry_get_text (GTK_ENTRY (tmp)), "") == 0)	{
					third_translator = 0;
				}
				else	{
					third_translator = get_translator_id (gtk_entry_get_text (GTK_ENTRY (tmp)));
				}
			}
			else	{
				if (strlen (volume_data->first_translator) == 0)	{
					first_translator = 0;
				}
				else	{
					first_translator = get_translator_id (volume_data->first_translator);
				}
				/*	Drugi tlumacz	*/
				if (strlen (volume_data->second_translator) == 0)	{
					second_translator = 0;
				}
				else	{
					second_translator = get_translator_id (volume_data->second_translator);
				}
				/*	Trzeci tlumacz	*/
				if (strlen (volume_data->third_translator) == 0)	{
					third_translator = 0;
				}
				else	{
					third_translator = get_translator_id (volume_data->third_translator);
				}
			}
		
			g_sprintf (sql_query, "UPDATE multi_volume_books_private_data SET "
				"title=\'%s\', "
				"orginal_title=\'%s\', "
				"isbn=\'%s\', "
				"publish_year=%d, "
				"pages=%d, "
				"first_translator=%d, "
				"second_translator=%d, "
				"third_translator=%d, "
				"description=\'%s\' "
		
				"WHERE "
		
				"id=%d;",
				volume_data->subtitle,
				volume_data->orginal_subtitle,
				volume_data->isbn,
				publish_year,
				volume_data->pages,
				first_translator,
				second_translator,
				third_translator,
				volume_data->description,
				volume_id);	
		
			/*	Aktualizujemy poszczegolne tomy	*/
			sqlite3_prepare (database, sql_query, -1, &db_row, 0);
			status = sqlite3_step (db_row);
			g_printf ("\t <<< Updating volume data with ID=%d, SQLite result: %d\n", volume_id, status);
			sqlite3_finalize (db_row);
		}
		
		/*	Pobieramy nastepny wiersz	*/
		result = gtk_tree_model_iter_next (model, &iter);
	}
}

/*
 *	Funkcja wyciaga wszystkie potrzebne dane o jednym tomie, ktorego id jest 
 *	wskazany poprzez argument 'volume_id'.
 */

MultiVolPrivDataStruc	*m_volume_select_data	(gint	*volume_id)	{
	MultiVolPrivDataStruc	*volume_data;
	gint					status;
	gchar					sql_query		[MEDIUM_STRING];
	sqlite3_stmt			*db_row;
	
	volume_data = g_new (MultiVolPrivDataStruc, 1);
	
	/*	Etap I - wyciagam wszystkie dane o tomie z tabeli 
	 *	'multi_volume_books_private_data'	*/
	
	g_sprintf (sql_query, "SELECT "
		"multi_id, "
		"title, "
		"orginal_title, "
		"isbn, "
		"publish_year, "
		"pages, "
		"first_translator, "
		"second_translator, "
		"third_translator, "
		"description "
		"FROM multi_volume_books_private_data "
		"WHERE id=%d;", *volume_id);
	
	sqlite3_prepare (database, sql_query, -1, &db_row, 0);
	status = sqlite3_step (db_row);
	g_printf ("<<< Selecting volume informations (1/2), SQLite result: %d\n", status);
	
	volume_data->id = *volume_id;
	volume_data->multi_id = sqlite3_column_int (db_row, 0);
	g_sprintf (volume_data->subtitle, "%s", sqlite3_column_text (db_row, 1));
	g_sprintf (volume_data->orginal_subtitle, "%s", sqlite3_column_text (db_row, 2));
	g_sprintf (volume_data->isbn, "%s", sqlite3_column_text (db_row, 3));
	/*	Pomijamy pole 'same_publish_year'	*/
	volume_data->publish_year = sqlite3_column_int (db_row, 4);
	volume_data->pages = sqlite3_column_int (db_row, 5);
	/*	Pomijamy pole 'same_translators'	*/
	volume_data->first_translator = sqlite3_column_int (db_row, 6);
	volume_data->second_translator = sqlite3_column_int (db_row, 7);
	volume_data->third_translator = sqlite3_column_int (db_row, 8);
	g_sprintf (volume_data->description, "%s", sqlite3_column_text (db_row, 9));
	
	sqlite3_finalize (db_row);	
	
	/*	Etap II - wyciagamy pola: 'same_publish_year' i 'same_translators'
	 *	z tabeli 'multi_volume_books_shared_data' i ewentualnie nadpisujemy
	 *	rok wydania i tlumaczy, o ile zostali ustawieni spolni dla wszystkich
	 *	tomow	*/
	 
	 g_sprintf (sql_query, "SELECT "
		"same_publish_year, "
		"publish_year, "
		
		"first_author, "
		"second_author, "
		"third_author, "
	
		"genere, "
		"publish_house, "
		
		"language, "
		"orginal_language, "
		
		"same_translators, "
		"first_translator, "
		"second_translator, "
		"third_translator, "
		
		"title, "
		"orginal_title "
		
		"FROM multi_volume_books_shared_data "
		"WHERE id=%d;", volume_data->multi_id);

	sqlite3_prepare (database, sql_query, -1, &db_row, 0);
	status = sqlite3_step (db_row);
	g_printf ("<<< Selecting volume informations (2/2), SQLite result: %d\n", status);
	
	volume_data->same_publish_year = sqlite3_column_int (db_row, 0);
	volume_data->same_translators = sqlite3_column_int (db_row, 9);
	
	volume_data->first_author = sqlite3_column_int (db_row, 2);
	volume_data->second_author = sqlite3_column_int (db_row, 3);
	volume_data->third_author = sqlite3_column_int (db_row, 4);
	volume_data->genere = sqlite3_column_int (db_row, 5);
	volume_data->publish_house = sqlite3_column_int (db_row, 6);
	volume_data->language = sqlite3_column_int (db_row, 7);
	volume_data->orginal_language = sqlite3_column_int (db_row, 8);
	
	if (volume_data->same_publish_year == 1	)	{
		/*	Nadpisujemy rok wydania tylko, gdy jest on wspolny dla wszystkich 
		 *	tomow	*/
		volume_data->publish_year = sqlite3_column_int (db_row, 1);
	}
	
	if (volume_data->same_translators == 1	)	{
		/*	Nadpisujemy tlumaczy tylko, gdy sa oni wspolni dla wszystkich 
		 *	tomow	*/
		volume_data->first_translator = sqlite3_column_int (db_row, 10);
		volume_data->second_translator = sqlite3_column_int (db_row, 11);
		volume_data->third_translator = sqlite3_column_int (db_row, 12);
	}
	
	g_sprintf (volume_data->title, "%s", sqlite3_column_text (db_row, 13));
	g_sprintf (volume_data->orginal_title, "%s", sqlite3_column_text (db_row, 14));

	sqlite3_finalize (db_row);
	
	return volume_data;
}

/*
 *	Funkcja wyswietla informacje o ksiazce w oknie 'property_win'
 */

void		m_volume_show_volume_data		(GtkWidget					*widget,
											 MultiVolPrivDataStruc	*volume_data)	{
	GladeXML			*xml;
	GtkTextBuffer		*text_buffer;
	GtkWidget			*tmp;
	gint				status;
	gchar				sql_query	[SHORT_STRING];
	gchar				string		[MEDIUM_STRING];
	gchar				item_name	[SHORT_STRING];
	sqlite3_stmt		*db_row;
	
	xml = glade_get_widget_tree (widget);
	
	g_printf ("<<< Displaying book's informations...\n");
	/*	Tworzymy tytul na podstawie szablonu: 
	 *	TYTUL_OGOLNY_KSIAZKI - TYTUL_TOMU
	 */
	g_sprintf (string, "%s - %s", volume_data->title, volume_data->subtitle);									 									 
	tmp = glade_xml_get_widget (xml, "title_lb");
	gtk_label_set_text (GTK_LABEL (tmp), string);
	
	/*	Wyszukujemy autorow po numerach id zwroconych w strukturze 'volume_data'	*/
	
	tmp = glade_xml_get_widget (xml, "author_lb");
	/*	Jezeli wszyscy autorzy sa ustawieni na 0, oznacza to, ze nie podano
		zadnego autora	*/
	if (volume_data->first_author == 0 &&
		volume_data->second_author == 0 &&
		volume_data->third_author == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (volume_data->first_author > 0)	{		
		get_author_name (item_name, volume_data->first_author);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
		
		if (volume_data->second_author > 0)	{
			get_author_name (item_name, volume_data->second_author);
			g_sprintf (string, "%s\n%s", gtk_label_get_text (GTK_LABEL (tmp)),
						item_name);
			gtk_label_set_text (GTK_LABEL (tmp), string);
		}
		if (volume_data->third_author > 0)	{
			get_author_name (item_name, volume_data->third_author);
			g_sprintf (string, "%s\n%s", gtk_label_get_text (GTK_LABEL (tmp)),
						item_name);
			gtk_label_set_text (GTK_LABEL (tmp), string);
		}
	}
	else	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	
	/*	Wyciagamy gatunek z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "genere_lb");
	/*	Jezeli gatunek ustawiony jest na 0, oznacza to, ze nie podano
		zadnego gatunku	*/
	if (volume_data->genere == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (volume_data->genere > 0)	{
		get_genere_name (item_name, volume_data->genere);		
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
	}
	
	/*	Wyciagamy wydawnictwo z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "publish_house_lb");
	/*	Jezeli wydawnictwo ustawione jest na 0, oznacza to, ze nie podano
		zadnego wydawnictwa	*/
	if (volume_data->publish_house == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (volume_data->publish_house > 0)	{
		get_publish_house_name (item_name, volume_data->publish_house);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
	}
	
	/*	Wyciagamy jezyk z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "language_lb");
	/*	Jezeli jezyk  ustawiony jest na 0, oznacza to, ze nie podano
		zadnego jezyka	*/
	if (volume_data->language == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (volume_data->language > 0)	{
		get_language_name (item_name, volume_data->language);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
	}
	
	/*	Tworzymy tytul na podstawie szablonu: 
	 *	ORGINALNY_TYTUL_OGOLNY_KSIAZKI - ORGINALNY_TYTUL_TOMU
	 */
	tmp = glade_xml_get_widget (xml, "orginal_title_lb");
	if (strlen (volume_data->orginal_subtitle) == 0)	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else	{
		if (strlen (volume_data->orginal_title) > 0)	{
			g_sprintf (string, "%s - %s", volume_data->orginal_title, volume_data->orginal_subtitle);
			gtk_label_set_text (GTK_LABEL (tmp), string);
		}
		else	{
			gtk_label_set_text (GTK_LABEL (tmp), volume_data->orginal_subtitle);
		}
	}
	
	/*	Wyciagamy orginalny jezyk z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "orginal_language_lb");
	/*	Jezeli orginalny jezyk  ustawiony jest na 0, oznacza to, ze nie podano
		zadnego jezyka	*/
	if (volume_data->orginal_language == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (volume_data->orginal_language > 0)	{
		get_language_name (item_name, volume_data->orginal_language);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
	}
	
	tmp = glade_xml_get_widget (xml, "isbn_lb");
	if (strlen (volume_data->isbn) == 0)	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else	{
		gtk_label_set_text (GTK_LABEL (tmp), volume_data->isbn);
	}
	
	tmp = glade_xml_get_widget (xml, "publish_year_lb");
	if (volume_data->publish_year == 0)	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else {
		g_sprintf (string, "%d", volume_data->publish_year);
		gtk_label_set_text (GTK_LABEL (tmp), string);
	}
	
	tmp = glade_xml_get_widget (xml, "pages_number_lb");
	if (volume_data->pages == 0)	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else {
		g_sprintf (string, "%d", volume_data->pages);
		gtk_label_set_text (GTK_LABEL (tmp), string);
	}
	
	/*	Wyszukujemy tlumaczy po numerach id zwroconych w strukturze 'volume_data'	*/
	
	tmp = glade_xml_get_widget (xml, "translator_lb");
	/*	Jezeli wszyscy tlumacze sa ustawieni na 0, oznacza to, ze nie podano
		zadnego tlimacza	*/
	if (volume_data->first_translator == 0 &&
		volume_data->second_translator == 0 &&
		volume_data->third_translator == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (volume_data->first_translator > 0)	{
		get_translator_name (item_name, volume_data->first_translator);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
		
		if (volume_data->second_translator > 0)	{
			get_translator_name (item_name, volume_data->second_translator);
			g_sprintf (string, "%s\n%s", gtk_label_get_text (GTK_LABEL (tmp)), item_name);
			gtk_label_set_text (GTK_LABEL (tmp), string);
		}
		
		if (volume_data->third_translator > 0)	{
			get_translator_name (item_name, volume_data->third_translator);
			g_sprintf (string, "%s\n%s", gtk_label_get_text (GTK_LABEL (tmp)), item_name);
			gtk_label_set_text (GTK_LABEL (tmp), string);
		}
	}
	else	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	
	tmp = glade_xml_get_widget (xml, "description_text");
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tmp));
	gtk_text_buffer_set_text (text_buffer, volume_data->description, strlen (volume_data->description));
	
	set_widget_sensinitive (tmp, "property_notebook", TRUE);
	
	g_free (volume_data);
}

/*
 *	Funkcja kontroluje ilosc otwartych elementow 'GtkExpander' woknie 'main_win',
 *	w zakladce wyswieltajacej informacje o ksiazce wielotomowej. W sytuacji, gdy
 *	otwarte sa wszystkie elemnty rozszerzajace wysokosc okna osiaga 
 *	wartosc ponad 1200 pikseli, co moze uniemozliwic uzywanie okna przy 
 *	mniejszych rozdzielczosciach ekranu. Ta funkcja kontroluje ilosc otwartych
 *	elementow rozszerzajacych w tym samym czasie, ograniczajac ja do 1
 *	otwartego elementu 'GtkExpander'.
 *
 *	Argument 'mode' oznacza, ze otwarty zostal element:
 *	- 'm_volume_additional_informations_exp'
 *	- 'm_volume_description_exp'
 */

void		m_volume_control_main_win_expanders	(gint	mode)	{
	GladeXML	*xml;
	GtkWidget	*m_volume_additional_informations_exp;
	GtkWidget	*m_volume_description_exp;
	
	xml = glade_get_widget_tree (main_win);
	m_volume_additional_informations_exp = glade_xml_get_widget (xml,
				"m_volume_additional_informations_exp");
	m_volume_description_exp = glade_xml_get_widget (xml, "m_volume_description_exp");
	
	switch (mode)	{
		case 0:
			gtk_expander_set_expanded (GTK_EXPANDER (m_volume_description_exp), FALSE);
			break;
		case 1:
			gtk_expander_set_expanded (GTK_EXPANDER (m_volume_additional_informations_exp), FALSE);
			break;
	}
}

/*
 *	Funkcja wyswietla informacje o wybranej ksiazce w oknie 'main_win'
 *	Przy okazji funkcja dokonuje malej obrobki wyswietlanych danych, wiecej
 *	informacji na ten temat w kodzie funkcji.
 */

void		m_volume_display_volume_data		(MultiVolPrivDataStruc	*volume_data)	{
	GladeXML			*xml;
	GtkTextBuffer		*text_buffer;
	GtkWidget			*tmp;
	gint				status;
	gchar				string		[SHORT_STRING];
	gchar				item_name	[SHORT_STRING];
	
	xml = glade_get_widget_tree (main_win);
	g_printf ("<<< Displaying book's informations...\n");
	
	set_widget_sensinitive (main_win, "main_m_volume_book_vbox", FALSE);
	
	/*	Wyswietlamy ID ksiazki	*/
	tmp = glade_xml_get_widget (xml, "m_volume_book_id_lb");
	g_sprintf (string, "%d", volume_data->id);
	gtk_label_set_text (GTK_LABEL (tmp), string);
	
	tmp = glade_xml_get_widget (xml, "m_volume_title_lb");
	gtk_label_set_text (GTK_LABEL (tmp), volume_data->title);
	
	tmp = glade_xml_get_widget (xml, "m_volume_subtitle_lb");
	gtk_label_set_text (GTK_LABEL (tmp), volume_data->subtitle);
	
	/*	Wyszukujemy autorow po numerach id zwroconych w strukturze 'volume_data'	*/
	
	tmp = glade_xml_get_widget (xml, "m_volume_author_lb");
	/*	Jezeli wszyscy autorzy sa ustawieni na 0, oznacza to, ze nie podano
		zadnego autora	*/
	if (volume_data->first_author == 0 &&
		volume_data->second_author == 0 &&
		volume_data->third_author == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (volume_data->first_author > 0)	{		
		get_author_name (item_name, volume_data->first_author);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
		
		if (volume_data->second_author > 0)	{
			get_author_name (item_name, volume_data->second_author);
			g_sprintf (string, "%s\n%s", gtk_label_get_text (GTK_LABEL (tmp)),
						item_name);
			gtk_label_set_text (GTK_LABEL (tmp), string);
		}
		if (volume_data->third_author > 0)	{
			get_author_name (item_name, volume_data->third_author);
			g_sprintf (string, "%s\n%s", gtk_label_get_text (GTK_LABEL (tmp)),
						item_name);
			gtk_label_set_text (GTK_LABEL (tmp), string);
		}
	}
	else	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	
	/*	Wyciagamy gatunek z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "m_volume_genere_lb");
	/*	Jezeli gatunek ustawiony jest na 0, oznacza to, ze nie podano
		zadnego gatunku	*/
	if (volume_data->genere == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (volume_data->genere > 0)	{
		get_genere_name (item_name, volume_data->genere);		
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
	}
	
	/*	Wyciagamy wydawnictwo z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "m_volume_publish_house_lb");
	/*	Jezeli wydawnictwo ustawione jest na 0, oznacza to, ze nie podano
		zadnego wydawnictwa	*/
	if (volume_data->publish_house == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (volume_data->publish_house > 0)	{
		get_publish_house_name (item_name, volume_data->publish_house);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
	}
	
	/*	Wyciagamy jezyk z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "m_volume_language_lb");
	/*	Jezeli jezyk  ustawiony jest na 0, oznacza to, ze nie podano
		zadnego jezyka	*/
	if (volume_data->language == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (volume_data->language > 0)	{
		get_language_name (item_name, volume_data->language);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
	}
	
	tmp = glade_xml_get_widget (xml, "m_volume_orginal_subtitle_lb");
	if (strlen (volume_data->orginal_subtitle) == 0)	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else	{
		gtk_label_set_text (GTK_LABEL (tmp), volume_data->orginal_title);
	}
	
	/*	Wyciagamy orginalny jezyk z bazy danych	*/
	
	tmp = glade_xml_get_widget (xml, "m_volume_orginal_language_lb");
	/*	Jezeli orginalny jezyk  ustawiony jest na 0, oznacza to, ze nie podano
		zadnego jezyka	*/
	if (volume_data->orginal_language == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (volume_data->orginal_language > 0)	{
		get_language_name (item_name, volume_data->orginal_language);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
	}
	
	tmp = glade_xml_get_widget (xml, "m_volume_isbn_lb");
	if (strlen (volume_data->isbn) == 0)	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else	{
		gtk_label_set_text (GTK_LABEL (tmp), volume_data->isbn);
	}
	
	tmp = glade_xml_get_widget (xml, "m_volume_publish_year_lb");
	if (volume_data->publish_year == 0)	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else {
		g_sprintf (string, "%d", volume_data->publish_year);
		gtk_label_set_text (GTK_LABEL (tmp), string);
	}
	
	tmp = glade_xml_get_widget (xml, "m_volume_pages_number_lb");
	if (volume_data->pages == 0)	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else {
		g_sprintf (string, "%d", volume_data->pages);
		gtk_label_set_text (GTK_LABEL (tmp), string);
	}
	
	/*	Wyszukujemy tlumaczy po numerach id zwroconych w strukturze 'volume_data'	*/
	
	tmp = glade_xml_get_widget (xml, "m_volume_translator_lb");
	/*	Jezeli wszyscy tlumacze sa ustawieni na 0, oznacza to, ze nie podano
		zadnego tlimacza	*/
	
	if (volume_data->first_translator == 0 &&
		volume_data->second_translator == 0 &&
		volume_data->third_translator == 0)
	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	else if (volume_data->first_translator > 0)	{
		get_translator_name (item_name, volume_data->first_translator);
		gtk_label_set_text (GTK_LABEL (tmp), item_name);
		
		if (volume_data->second_translator > 0)	{
			get_translator_name (item_name, volume_data->second_translator);
			g_sprintf (string, "%s\n%s", gtk_label_get_text (GTK_LABEL (tmp)), item_name);
			gtk_label_set_text (GTK_LABEL (tmp), string);
		}
		
		if (volume_data->third_translator > 0)	{
			get_translator_name (item_name, volume_data->third_translator);
			g_sprintf (string, "%s\n%s", gtk_label_get_text (GTK_LABEL (tmp)), item_name);
			gtk_label_set_text (GTK_LABEL (tmp), string);
		}
	}
	else	{
		gtk_label_set_text (GTK_LABEL (tmp), _("Unknown"));
	}
	
	tmp = glade_xml_get_widget (xml, "m_volume_description_text");
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tmp));
	gtk_text_buffer_set_text (text_buffer, volume_data->description, strlen (volume_data->description));
	
	set_widget_sensinitive (main_win, "main_m_volume_book_vbox", TRUE);
	g_free (volume_data);
}

/*
 *	Funkcja usuwa pozycje z tabeli '' o wskazanym id 'volume_id'. Ksiazka
 *	wielotomowa moze zawierac minimalnie 2 tomy.
 */

void	m_volume_remove_volume	(gint	*volume_id)	{
	gint				status;
	gint				result;
	gint				multi_id;
	gchar				sql_query	[SHORT_STRING];
	sqlite3_stmt		*db_row;
	
	/*	Ustalamy ID ksiazki wielotomowej	*/
	g_sprintf (sql_query, "SELECT multi_id FROM multi_volume_books_private_data WHERE id=%d;", *volume_id);
	sqlite3_prepare (database, sql_query, -1, &db_row, 0);
	status = sqlite3_step (db_row);
	g_printf ("<<< Getting multi volume book ID, SQLite result: %d\n", status);
	multi_id = sqlite3_column_int (db_row, 0);
	sqlite3_finalize (db_row);
	
	/*	Ustalamy ID ksiazki wielotomowej	*/
	g_sprintf (sql_query, "SELECT COUNT(id) FROM multi_volume_books_private_data WHERE multi_id=%d;", multi_id);
	sqlite3_prepare (database, sql_query, -1, &db_row, 0);
	status = sqlite3_step (db_row);
	g_printf ("<<< Counting voumes, SQLite result: %d\n", status);
	result = sqlite3_column_int (db_row, 0);
	sqlite3_finalize (db_row);
	
	if (result > 2)	{
		g_sprintf (sql_query, "DELETE FROM multi_volume_books_private_data WHERE id=%d;", *volume_id);
		sqlite3_prepare (database, sql_query, -1, &db_row, 0);
		status = sqlite3_step (db_row);
		g_printf ("<<< Removing volume of multi volume book from database, SQLite result: %d\n", status);
		sqlite3_finalize (db_row);
	}
	else	{
		show_message_dlg (_("Can\'t remove item"), _("Multi volume book must contain minumum two volumes!"), 2);
	}
}

/*
 *	Funkcja czysci wszytskie pola wyswieltajace informacje o ksiazkach
 *	jednotomowych w oknie 'main_win' i dezaktywuje przyciski odpowiedzialne
 *	za edycje, usuwanie itp. ksiazki jednotomowej o oknie 'main_wion'.
 *	Funkcja jest przydatna np. gdy usuniemy ksiazke z bazy danych.
 */

void				m_volume_clear_main_win		(void)	{
	GladeXML			*xml;
	GtkTextBuffer		*text_buffer;
	GtkWidget			*tmp;
	
	xml = glade_get_widget_tree (main_win);
	tmp = glade_xml_get_widget (xml, "m_volume_book_id_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_volume_title_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_volume_subtitle_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_volume_author_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_volume_genere_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_volume_publish_house_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_volume_language_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_volume_orginal_subtitle_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_volume_orginal_language_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_volume_isbn_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_volume_publish_year_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_volume_pages_number_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_volume_translator_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = glade_xml_get_widget (xml, "m_volume_description_text");
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tmp));
	gtk_text_buffer_set_text (text_buffer, "", 0);
	
	set_widget_sensinitive (tmp, "m_volume_remove_but", FALSE);
}
